@@ -34,7 +34,12 @@ Link &NetLatch::pin_Q( unsigned w )
 Link &NetLatch::pin_Gate()
 {
   return pin( 0 );
-} // end function pin_Q
+} // end function pin_Gate
+
+const Link &NetLatch::pin_Gate() const
+{
+  return pin( 0 );
+} // end function pin_Gate
 
 bool NetLatch::emit_node( target_t *driverPtr ) const
 {
