@@ -19,7 +19,7 @@
  *    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
  */
 #ifdef HAVE_CVS_IDENT
-#ident "$Id: compiler.h,v 1.26 2004/10/04 01:10:52 steve Exp $"
+#ident "$Id: compiler.h,v 1.26.2.3 2006/06/12 00:16:50 steve Exp $"
 #endif
 
 # include  <list>
@@ -78,12 +78,17 @@ extern bool warn_timescale;
 /* Warn about legal but questionable module port bindings. */
 extern bool warn_portbinding;
 
+/* Warn about unused or unassigned variables. */
+extern bool warn_unused;
+
 /* This is true if verbose output is requested. */
 extern bool verbose_flag;
 
 extern bool debug_scopes;
 extern bool debug_eval_tree;
 extern bool debug_elaborate;
+extern bool debug_synth;
+extern bool debug_cprop;
 
 /* Path to a directory useful for finding subcomponents. */
 extern const char*basedir;
@@ -136,6 +141,15 @@ extern int load_sys_func_table(const char*path);
 
 /*
  * $Log: compiler.h,v $
+ * Revision 1.26.2.3  2006/06/12 00:16:50  steve
+ *  Add support for -Wunused warnings.
+ *
+ * Revision 1.26.2.2  2006/04/23 04:25:45  steve
+ *  Add cprop debugging.
+ *
+ * Revision 1.26.2.1  2006/04/01 01:37:24  steve
+ *  Add synth debug flag
+ *
  * Revision 1.26  2004/10/04 01:10:52  steve
  *  Clean up spurious trailing white space.
  *
