@@ -17,7 +17,7 @@
  *    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
  */
 #ifdef HAVE_CVS_IDENT
-#ident "$Id: main.c,v 1.65 2004/06/17 14:47:22 steve Exp $"
+#ident "$Id: main.c,v 1.65.2.6 2007/05/30 17:48:26 steve Exp $"
 #endif
 
 # include "config.h"
@@ -88,6 +88,10 @@ extern const char*optarg;
 # define IVL_ROOT "."
 #endif
 
+#ifndef IVL_ROOT_VARIABLE
+# define IVL_ROOT_VARIABLE "IVERILOG_ROOT"
+#endif
+
 # include  "globals.h"
 #include "cfparse_misc.h"   /* cfparse() */
 
@@ -262,12 +266,66 @@ static int t_default(char*cmd, unsigned ncmd)
       }
 #endif
 
+#ifdef DEBUG
+      char *original_cmd = malloc( strlen( cmd ) + (size_t)1 );
+      strcpy( original_cmd, cmd );
+
+      cmd = realloc( cmd, strlen( original_cmd ) + (size_t)19 ); /* more than enough memory for the ivlpp command and ">/tmp/ivlpp_stdout" */
+      if( cmd == NULL ){
+	exit( EXIT_FAILURE );
+      }
+
+      char *debug_ivl = malloc( strlen( original_cmd ) + (size_t)5 ); /* more than enough memory for "gdb " and ivl program_name */
+      strcpy( debug_ivl, "gdb " );
+
+      static char whitespace[] = " \t\f\r\v\n";
+      char *token;
+      int found_pipe = 0;
+      unsigned long token_counter = 0UL;
+
+      strcpy( cmd, "" );
+      for( token = strtok( original_cmd, whitespace ); token != NULL; token = strtok( NULL, whitespace ) ){
+
+	if( strcmp( token, "|" ) == 0 ){
+	  found_pipe = 1;
+	}
+
+	if( found_pipe ){
+	  token_counter++;
+	}
+
+	if( token_counter == 0UL ){
+	  strcat( cmd, token );
+	  strcat( cmd, " " ); /* harmless space after last token, and it was already there anyway */
+	}
+
+	if( token_counter == 2UL ){
+	  strcat( debug_ivl, token );
+	}
+
+	if( token_counter > 2UL ){
+	  printf( "%s\n", token );
+	}
+
+      }
+
+      free( original_cmd );
+      strcat( cmd, ">/tmp/ivlpp_stdout" );
+#endif
 
       if (verbose_flag)
 	    printf("translate: %s\n", cmd);
 
 
       rc = system(cmd);
+#ifdef DEBUG
+      if( verbose_flag ){
+	printf( "%s\n", debug_ivl );
+      }
+
+      system( debug_ivl ); /* For simplicity, ignore any return value, which is implementation dependent anyway. */
+      free( debug_ivl );
+#endif
       remove(source_path);
       if ( ! getenv("IVERILOG_ICONFIG"))
 	    remove(iconfig_path);
@@ -291,7 +349,7 @@ static int t_default(char*cmd, unsigned ncmd)
 static void process_warning_switch(const char*name)
 {
       if (strcmp(name,"all") == 0) {
-	    strcat(warning_flags, "ipt");
+	    strcat(warning_flags, "iptu");
 
       } else if (strcmp(name,"implicit") == 0) {
 	    if (! strchr(warning_flags+2, 'i'))
@@ -302,6 +360,9 @@ static void process_warning_switch(const char*name)
       } else if (strcmp(name,"timescale") == 0) {
 	    if (! strchr(warning_flags+2, 't'))
 		  strcat(warning_flags, "t");
+      } else if (strcmp(name,"unused") == 0) {
+	    if (! strchr(warning_flags+2, 'u'))
+		  strcat(warning_flags, "u");
       } else if (strcmp(name,"no-implicit") == 0) {
 	    char*cp = strchr(warning_flags+2, 'i');
 	    if (cp) while (*cp) {
@@ -320,6 +381,12 @@ static void process_warning_switch(const char*name)
 		  cp[0] = cp[1];
 		  cp += 1;
 	    }
+      } else if (strcmp(name,"no-unused") == 0) {
+	    char*cp = strchr(warning_flags+2, 'u');
+	    if (cp) while (*cp) {
+		  cp[0] = cp[1];
+		  cp += 1;
+	    }
       }
 }
 
@@ -409,14 +476,13 @@ int process_generation(const char*name)
       return 0;
 }
 
-int main(int argc, char **argv)
+/*
+ * This function fills in th ivl_root directory with the appropriate
+ * path, based on operating system, build time configuration, and
+ * environment.
+ */
+static char* get_root_dir()
 {
-      char*cmd;
-      unsigned ncmd;
-      int e_flag = 0;
-      int version_flag = 0;
-      int opt, idx, rc;
-
 #ifdef __MINGW32__
       { char * s;
 	char basepath[1024];
@@ -443,16 +509,56 @@ int main(int argc, char **argv)
 	s = strrchr(ivl_root, sep);
 	if (s) *s = 0;
 	strcat(ivl_root, "\\lib\\ivl");
-
-	base = ivl_root;
       }
-
 #else
-        /* In a UNIX environment, the IVL_ROOT from the Makefile is
-	   dependable. It points to the $prefix/lib/ivl directory,
-	   where the sub-parts are installed. */
-      strcpy(ivl_root, IVL_ROOT);
-      base = ivl_root;
+
+      { char*var = 0;
+
+	  /* In other systems, use the configured root. */
+        strcpy(ivl_root, IVL_ROOT);
+
+	  /* In any case, the IVL_ROOT variable can be used to override
+	     the install location at run time. If it is set, use that
+	     value instead. */
+	if ( (var = getenv(IVL_ROOT_VARIABLE)) ) {
+	      strncpy(ivl_root, var, MAXSIZE);
+	      strcat(ivl_root, "/lib/ivl");
+	}
+      }
+#endif
+
+      return ivl_root;
+}
+
+int main(int argc, char **argv)
+{
+      char*cmd;
+      unsigned ncmd;
+      int e_flag = 0;
+      int version_flag = 0;
+      int opt, idx, rc;
+
+      base = get_root_dir();
+
+#ifdef __MINGW32__
+	/* Under Windows, there is no point checking the root
+	   directory. */
+#else
+	/* On other systems, check that the root directory seems
+	   reasonable. */
+      { size_t tmp_len = strlen(base) + 16;
+        char*  tmp = malloc(tmp_len);
+	snprintf(tmp, tmp_len, "%s%civl", base, sep);
+
+	if (0 != access(tmp, F_OK)) {
+	      fprintf(stderr, "%s: INSTALLATION ERROR: Directory %s"
+		      " does not contain ivl files.\n", argv[0], base);
+	      fprintf(stderr, "%s:                   : Check installation"
+		      " or set " IVL_ROOT_VARIABLE " correctly.\n", argv[0]);
+	      return 1;
+	}
+	free(tmp);
+      }
 #endif
 
 	/* Create a temporary file for communicating input parameters
@@ -594,7 +700,7 @@ int main(int argc, char **argv)
 	    pbase = base;
 
       if (version_flag || verbose_flag) {
-	    printf("Icarus Verilog version " VERSION " ($Name: v0_8 $)\n");
+	    printf("Icarus Verilog version " VERSION " ($Name:  $)\n");
 	    printf("Copyright 1998-2003 Stephen Williams\n");
 	    puts(NOTICE);
 
@@ -735,6 +841,24 @@ int main(int argc, char **argv)
 
 /*
  * $Log: main.c,v $
+ * Revision 1.65.2.6  2007/05/30 17:48:26  steve
+ *  DEBUG aids. (Alan Feldstein)
+ *
+ * Revision 1.65.2.5  2006/07/07 21:31:50  steve
+ *  Root dir variable does not include lib/ivl components.
+ *
+ * Revision 1.65.2.4  2006/06/27 01:30:20  steve
+ *  Fix unused var warning for mingw32 build.
+ *
+ * Revision 1.65.2.3  2006/06/14 03:01:49  steve
+ *  Remove redundant call to get_root_dir.
+ *
+ * Revision 1.65.2.2  2006/06/12 00:16:53  steve
+ *  Add support for -Wunused warnings.
+ *
+ * Revision 1.65.2.1  2006/03/26 21:47:26  steve
+ *  More installation directory flexibility.
+ *
  * Revision 1.65  2004/06/17 14:47:22  steve
  *  Add a .sft file for the system functions.
  *
