@@ -17,7 +17,7 @@
  *    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
  */
 #ifdef HAVE_CVS_IDENT
-#ident "$Id: elab_net.cc,v 1.138 2004/10/04 03:09:38 steve Exp $"
+#ident "$Id: elab_net.cc,v 1.138.2.3 2005/09/11 02:56:37 steve Exp $"
 #endif
 
 # include "config.h"
@@ -1216,6 +1216,13 @@ NetNet* PECallFunction::elaborate_net(Design*des, NetScope*scope,
 	    return sub;
       }
 
+      if (path_.peek_name(0)[0] == '$') {
+	    cerr << get_line() << ": sorry: System functions " << path_ <<
+		  " not supported in continuous assignments." << endl;
+	    des->errors += 1;
+	    return 0;
+      }
+
 	/* Look up the function definition. */
       NetFuncDef*def = des->find_function(scope, path_);
       if (def == 0) {
@@ -1312,21 +1319,25 @@ NetNet* PEConcat::elaborate_net(Design*des, NetScope*scope,
       unsigned repeat = 1;
 
       if (repeat_) {
-	    verinum*rep = repeat_->eval_const(des, scope);
-	    if (rep == 0) {
+	    NetExpr*etmp = elab_and_eval(des, scope, repeat_);
+	    assert(etmp);
+	    NetEConst*erep = dynamic_cast<NetEConst*>(etmp);
+
+	    if (erep == 0) {
 		  cerr << get_line() << ": internal error: Unable to "
 		       << "evaluate constant repeat expression." << endl;
 		  des->errors += 1;
 		  return 0;
 	    }
 
-	    repeat = rep->as_ulong();
+	    repeat = erep->value().as_ulong();
+	    delete etmp;
 
 	    if (repeat == 0) {
-		  cerr << get_line() << ": error: Invalid repeat value."
+		  cerr << get_line() << ": error: Concatenation repeat "
+			"may not be 0."
 		       << endl;
 		  des->errors += 1;
-		  delete rep;
 		  return 0;
 	    }
       }
@@ -1435,6 +1446,7 @@ NetNet* PEIdent::elaborate_net_bitmux_(Design*des, NetScope*scope,
 
       NetMux*mux = new NetMux(scope, scope->local_symbol(), 1,
 			      sig_width, sel->pin_count());
+      mux->set_line(*this);
 
 	/* Connect the signal bits to the mux. Account for the
 	   direction of the numbering (lsb to msb vs. msb to lsb) by
@@ -2268,6 +2280,7 @@ NetNet* PETernary::elaborate_net(Design*des, NetScope*scope,
 	   (true) connected to tru_sig.  */
 
       NetMux*mux = new NetMux(scope, scope->local_symbol(), dwidth, 2, 1);
+      mux->set_line(*this);
       connect(mux->pin_Sel(0), expr_sig->pin(0));
 
 	/* Connect the data inputs. */
@@ -2413,6 +2426,9 @@ NetNet* PEUnary::elaborate_net(Design*des, NetScope*scope,
 	    reduction=true; gtype = NetLogic::XNOR; break;
 
 	  case '-': // Unary 2's complement.
+	    if (owidth == 0)
+		  owidth = sub_sig->pin_count();
+
 	    sig = new NetNet(scope, scope->local_symbol(),
 			     NetNet::WIRE, owidth);
 	    sig->local_flag(true);
@@ -2459,15 +2475,15 @@ NetNet* PEUnary::elaborate_net(Design*des, NetScope*scope,
 
 		default:
 		  NetAddSub*sub = new NetAddSub(scope, scope->local_symbol(),
-						sig->pin_count());
+						owidth);
 		  sub->attribute(perm_string::literal("LPM_Direction"), verinum("SUB"));
 
 		  des->add_node(sub);
 
-		  for (unsigned idx = 0 ;  idx < sig->pin_count();  idx += 1)
+		  for (unsigned idx = 0 ;  idx < owidth ;  idx += 1)
 			connect(sig->pin(idx), sub->pin_Result(idx));
 
-		  for (unsigned idx = 0; idx < sub_sig->pin_count(); idx += 1)
+		  for (unsigned idx = 0; idx < owidth; idx += 1)
 			connect(sub_sig->pin(idx), sub->pin_DataB(idx));
 
 		  verinum tmp_num (verinum::V0, sub->width(), true);
@@ -2477,8 +2493,7 @@ NetNet* PEUnary::elaborate_net(Design*des, NetScope*scope,
 		  des->add_node(tmp_con);
 
 		  NetNet*tmp_sig = new NetNet(scope, scope->local_symbol(),
-					      NetNet::WIRE,
-					      sub_sig->pin_count());
+					      NetNet::WIRE, owidth);
 		  tmp_sig->local_flag(true);
 
 		  for (unsigned idx = 0; idx < sig->pin_count(); idx += 1) {
@@ -2513,6 +2528,15 @@ NetNet* PEUnary::elaborate_net(Design*des, NetScope*scope,
 
 /*
  * $Log: elab_net.cc,v $
+ * Revision 1.138.2.3  2005/09/11 02:56:37  steve
+ *  Attach line numbers to NetMux devices.
+ *
+ * Revision 1.138.2.2  2005/02/19 16:39:30  steve
+ *  Spellig fixes.
+ *
+ * Revision 1.138.2.1  2005/01/29 00:18:23  steve
+ *  Fix evaluate of constants in netlist concatenation repeats.
+ *
  * Revision 1.138  2004/10/04 03:09:38  steve
  *  Fix excessive error message.
  *
