@@ -469,6 +469,12 @@ void PGBuiltin::elaborate(Design*des, NetScope*scope) const
 
       for (unsigned idx = 0 ;  idx < pin_count() ;  idx += 1) {
 	    const PExpr*ex = pin(idx);
+	    if (ex == 0) {
+		  cerr << get_line() << ": error: Logic gate port "
+			"expressions are not optional." << endl;
+		  des->errors += 1;
+		  return;
+	    }
 	    NetNet*sig = (idx == 0)
 		  ? ex->elaborate_lnet(des, scope, true)
 		  : ex->elaborate_net(des, scope, 0, 0, 0, 0);
@@ -1964,7 +1970,12 @@ NetProc* PEventStatement::elaborate_st(Design*des, NetScope*scope,
 
       if (expr_.count() == 0) {
 	    assert(enet);
-	    NexusSet*nset = enet->nex_input();
+	     /* For synthesis we want just the inputs, but for the rest we
+	      * want inputs and outputs that may cause a value to change. */
+	    extern bool synthesis; /* Synthesis flag from main.cc */
+	    bool rem_out = false;
+	    if (synthesis) rem_out = true;
+	    NexusSet*nset = enet->nex_input(rem_out);
 	    if (nset == 0) {
 		  cerr << get_line() << ": internal error: No NexusSet"
 		       << " from statement." << endl;
@@ -2356,6 +2367,13 @@ NetProc* PForStatement::elaborate(Design*des, NetScope*scope) const
 	   the for loop. This is also a very specific assignment
 	   statement. Put this into the "body" block. */
       sig = des->find_signal(scope, id2->path());
+      if (sig == 0) {
+	    cerr << get_line() << ": error: Unable to find variable "
+		 << id2->path() << " in for-loop increment expression." << endl;
+	    des->errors += 1;
+	    return body;
+      }
+
       assert(sig);
       lv = new NetAssign_(sig);
 
