@@ -17,7 +17,7 @@
  *    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
  */
 #ifdef HAVE_CVS_IDENT
-#ident "$Id: net_assign.cc,v 1.18 2004/08/28 15:08:31 steve Exp $"
+#ident "$Id: net_assign.cc,v 1.18.2.3 2006/05/02 02:00:15 steve Exp $"
 #endif
 
 # include "config.h"
@@ -45,6 +45,7 @@ NetAssign_::NetAssign_(NetNet*s)
       lwid_ = sig_->pin_count();
       sig_->incr_lref();
       more = 0;
+      mem_lref_ = false;
 }
 
 NetAssign_::NetAssign_(NetMemory*s)
@@ -53,6 +54,7 @@ NetAssign_::NetAssign_(NetMemory*s)
       loff_ = 0;
       lwid_ = mem_->width();
       more = 0;
+      mem_lref_ = false;
 }
 
 NetAssign_::NetAssign_(NetVariable*s)
@@ -61,6 +63,7 @@ NetAssign_::NetAssign_(NetVariable*s)
       loff_ = 0;
       lwid_ = 0;
       more = 0;
+      mem_lref_ = false;
 }
 
 NetAssign_::~NetAssign_()
@@ -71,6 +74,14 @@ NetAssign_::~NetAssign_()
 		  sig_->type(NetNet::WIRE);
       }
 
+      if (mem_) {
+	    NetNet*exp = mem_->reg_from_explode();
+	    if (exp && mem_lref_) {
+		  exp->decr_lref();
+		  if (turn_sig_to_wire_on_release_ && exp->peek_lref() == 0)
+			exp->type(NetNet::WIRE);
+	    }
+      }
       assert( more == 0 );
       if (bmux_) delete bmux_;
 }
@@ -111,7 +122,10 @@ perm_string NetAssign_::name() const
 
 NetNet* NetAssign_::sig() const
 {
-      return sig_;
+      if (mem_)
+	    return 0;
+      else
+	    return sig_;
 }
 
 NetMemory* NetAssign_::mem() const
@@ -124,6 +138,16 @@ NetVariable* NetAssign_::var() const
       return var_;
 }
 
+void NetAssign_::incr_mem_lref()
+{
+      if (! mem_lref_) {
+	    assert(mem_);
+	    NetNet*exp = mem_->reg_from_explode();
+	    assert(exp);
+	    exp->incr_lref();
+	    mem_lref_ = true;
+      }
+}
 
 void NetAssign_::set_part(unsigned lo, unsigned lw)
 {
@@ -261,6 +285,15 @@ NetAssignNB::~NetAssignNB()
 
 /*
  * $Log: net_assign.cc,v $
+ * Revision 1.18.2.3  2006/05/02 02:00:15  steve
+ *  Fix uninitialized mem_lref_ member.
+ *
+ * Revision 1.18.2.2  2006/04/16 19:26:38  steve
+ *  Fix handling of exploded memories with partial or missing resets.
+ *
+ * Revision 1.18.2.1  2006/03/12 07:34:17  steve
+ *  Fix the memsynth1 case.
+ *
  * Revision 1.18  2004/08/28 15:08:31  steve
  *  Do not change reg to wire in NetAssign_ unless synthesizing.
  *
