@@ -17,7 +17,7 @@
  *    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
  */
 #ifdef HAVE_CVS_IDENT
-#ident "$Id: net_link.cc,v 1.14 2004/02/18 17:11:56 steve Exp $"
+#ident "$Id: net_link.cc,v 1.14.2.6 2006/08/23 04:09:14 steve Exp $"
 #endif
 
 # include "config.h"
@@ -40,30 +40,33 @@ void connect(Nexus*l, Link&r)
       if (l == r.nexus_)
 	    return;
 
-
       Nexus*tmp = r.nexus_;
+
       while (Link*cur = tmp->list_) {
 	    tmp->list_ = cur->next_;
 	    cur->nexus_ = 0;
 	    cur->next_  = 0;
+	    cur->prev_  = 0;
 	    l->relink(cur);
       }
 
-      l->driven_ = Nexus::NO_GUESS;
-
-      assert(tmp->list_ == 0);
       delete tmp;
 }
 
 void connect(Link&l, Link&r)
 {
       assert(&l != &r);
-      connect(l.nexus_, r);
+      if (r.is_linked() && !l.is_linked())
+	    connect(r.nexus_, l);
+      else if (r.nexus_->list_len_ > l.nexus_->list_len_)
+	    connect(r.nexus_, l);
+      else
+	    connect(l.nexus_, r);
 }
 
 Link::Link()
 : dir_(PASSIVE), drive0_(STRONG), drive1_(STRONG), init_(verinum::Vx),
-  inst_(0), next_(0), nexus_(0)
+  inst_(0), next_(0), prev_(0), nexus_(0)
 {
       (new Nexus()) -> relink(this);
 }
@@ -215,6 +218,7 @@ Nexus::Nexus()
 {
       name_ = 0;
       list_ = 0;
+      list_len_ = 0;
       driven_ = NO_GUESS;
       t_cookie_ = 0;
 }
@@ -241,6 +245,18 @@ verinum::V Nexus::get_init() const
       return verinum::Vz;
 }
 
+int Nexus::is_driven() const
+{
+      int count = 0;
+
+      for (Link*cur = list_ ; cur ;  cur = cur->next_) {
+	    if (cur->get_dir() == Link::OUTPUT)
+		  count += 1;
+      }
+
+      return count;
+}
+
 void Nexus::unlink(Link*that)
 {
       if (name_) {
@@ -254,22 +270,32 @@ void Nexus::unlink(Link*that)
 	    driven_ = NO_GUESS;
 
       assert(that);
+      assert(that->nexus_ == this);
       if (list_ == that) {
+	    assert(that->prev_ == 0);
+	      // Move the second item the front of the list.
 	    list_ = that->next_;
+	    if (list_)
+		  list_->prev_ = 0;
+	    list_len_ -= 1;
+	      // Clear the pointers for the link.
 	    that->next_ = 0;
+	    that->prev_ = 0;
 	    that->nexus_ = 0;
 	    return;
       }
 
-      Link*cur = list_;
-      while (cur->next_ != that) {
-	    assert(cur->next_);
-	    cur = cur->next_;
-      }
+
+      Link*cur = that->prev_;
+      assert(cur->nexus_ == this);
 
       cur->next_ = that->next_;
+      if (cur->next_)
+	    cur->next_->prev_ = cur;
       that->nexus_ = 0;
       that->next_ = 0;
+      that->prev_ = 0;
+      list_len_ -= 1;
 }
 
 void Nexus::relink(Link*that)
@@ -287,8 +313,13 @@ void Nexus::relink(Link*that)
       assert(that->nexus_ == 0);
       assert(that->next_ == 0);
       that->next_ = list_;
+      that->prev_ = 0;
+      if (that->next_)
+	    that->next_->prev_ = that;
+
       that->nexus_ = this;
       list_ = that;
+      list_len_ += 1;
 }
 
 Link* Nexus::first_nlink()
@@ -360,8 +391,9 @@ const char* Nexus::name() const
 		  obj->name() << " pin " << pin << "(" <<
 		  lnk->get_name() << "<" << lnk->get_inst() << ">)"
 		  " type=" << typeid(*obj).name() << "?" << endl;
-
+	    return 0;
       }
+
       assert(sig);
       ostringstream tmp;
       tmp << sig->name();
@@ -378,6 +410,7 @@ const char* Nexus::name() const
 NexusSet::NexusSet()
 {
       items_ = 0;
+      index_ = 0;
       nitems_ = 0;
 }
 
@@ -386,8 +419,11 @@ NexusSet::~NexusSet()
       if (nitems_ > 0) {
 	    assert(items_ != 0);
 	    delete[] items_;
+	    assert(index_ != 0);
+	    delete[] index_;
       } else {
 	    assert(items_ == 0);
+	    assert(index_ == 0);
       }
 }
 
@@ -396,25 +432,44 @@ unsigned NexusSet::count() const
       return nitems_;
 }
 
+/*
+ * Add the Nexus to the nexus set at the *end* of the array. This
+ * preserves order, which is used in a few cases by the
+ * synthesizer. But for efficiency, also create a sorted index.
+ */
 void NexusSet::add(Nexus*that)
 {
+	/* Handle the special case that the set is empty. */
       if (nitems_ == 0) {
 	    assert(items_ == 0);
+	    assert(index_ == 0);
 	    items_ = (Nexus**)malloc(sizeof(Nexus*));
 	    items_[0] = that;
+	    index_ = (unsigned*)malloc(sizeof(unsigned));
+	    index_[0] = 0;
 	    nitems_ = 1;
 	    return;
       }
 
       unsigned ptr = bsearch_(that);
-      if ((ptr < nitems_) && (items_[ptr] == that))
+      if (ptr < nitems_ && items_[index_[ptr]] == that) {
+	    assert(items_[index_[ptr]] == that);
 	    return;
+      }
 
-      items_ = (Nexus**)realloc(items_, (nitems_+1) * sizeof(Nexus*));
-      for (unsigned idx = nitems_ ;  idx > ptr ;  idx -= 1)
-	    items_[idx] = items_[idx-1];
+      items_ = (Nexus**)  realloc(items_, (nitems_+1) * sizeof(Nexus*));
+      index_ = (unsigned*)realloc(index_, (nitems_+1) * sizeof(unsigned));
+
+      items_[nitems_] = that;
+
+      unsigned dest = nitems_;
+      for (unsigned idx = ptr ;  idx < nitems_ ;  idx += 1) {
+	    unsigned tmp = index_[idx];
+	    index_[idx] = dest;
+	    dest = tmp;
+      }
+      index_[nitems_] = dest;
 
-      items_[ptr] = that;
       nitems_ += 1;
 }
 
@@ -430,20 +485,31 @@ void NexusSet::rem(Nexus*that)
 	    return;
 
       unsigned ptr = bsearch_(that);
-      if ((ptr >= nitems_) || (items_[ptr] != that))
+      if (ptr >= nitems_ || items_[index_[ptr]] != that)
 	    return;
 
       if (nitems_ == 1) {
 	    free(items_);
+	    free(index_);
 	    items_ = 0;
+	    index_ = 0;
 	    nitems_ = 0;
 	    return;
       }
 
-      for (unsigned idx = ptr ;  idx < (nitems_-1) ;  idx += 1)
+      unsigned index_ptr = index_[ptr];
+      for (unsigned idx = index_ptr ;  idx < (nitems_-1) ;  idx += 1)
 	    items_[idx] = items_[idx+1];
 
-      items_ = (Nexus**)realloc(items_, (nitems_-1) * sizeof(Nexus*));
+      for (unsigned idx = ptr ;  idx < (nitems_-1) ;  idx += 1)
+	    index_[idx] = index_[idx+1];
+
+      for (unsigned idx = 0 ;  idx < (nitems_-1) ;  idx += 1)
+	    if (index_[idx] > index_ptr)
+		  index_[idx] -= 1;
+
+      items_ = (Nexus**)  realloc(items_, (nitems_-1) * sizeof(Nexus*));
+      index_ = (unsigned*)realloc(index_, (nitems_-1) * sizeof(unsigned));
       nitems_ -= 1;
 }
 
@@ -459,16 +525,37 @@ Nexus* NexusSet::operator[] (unsigned idx) const
       return items_[idx];
 }
 
+/*
+ * This method uses binary search to locate the item in the list of
+ * nexus pointers. If the item is in the set, then this method returns
+ * the index where it exists in the *index* array. If the item is not
+ * in the set, the index points to where in the array the item should go.
+ */
 unsigned NexusSet::bsearch_(Nexus*that) const
 {
-      for (unsigned idx = 0 ;  idx < nitems_ ;  idx += 1) {
-	    if (items_[idx] < that)
-		  continue;
 
-	    return idx;
+      unsigned low = 0, hig = nitems_;
+
+      while (low < hig) {
+	    unsigned mid = (low + hig) / 2;
+	    if (mid == hig) mid -= 1;
+	    assert(mid >= low);
+	    assert(mid < hig);
+
+	    if (items_[index_[mid]] == that) {
+		  return mid;
+
+	    } else if (items_[index_[mid]] > that) {
+		  hig = mid;
+
+	    } else {
+		  low = mid+1;
+	    }
       }
 
-      return nitems_;
+      assert(low == hig);
+      assert(low == nitems_ || items_[index_[low]] >= that);
+      return low;
 }
 
 bool NexusSet::contains(const NexusSet&that) const
@@ -477,7 +564,7 @@ bool NexusSet::contains(const NexusSet&that) const
 	    unsigned where = bsearch_(that[idx]);
 	    if (where == nitems_)
 		  return false;
-	    if (items_[where] != that[idx])
+	    if (items_[index_[where]] != that[idx])
 		  return false;
       }
 
@@ -490,7 +577,7 @@ bool NexusSet::intersect(const NexusSet&that) const
 	    unsigned where = bsearch_(that[idx]);
 	    if (where == nitems_)
 		  continue;
-	    if (items_[where] == that[idx])
+	    if (items_[index_[where]] == that[idx])
 		  return true;
       }
 
@@ -499,6 +586,24 @@ bool NexusSet::intersect(const NexusSet&that) const
 
 /*
  * $Log: net_link.cc,v $
+ * Revision 1.14.2.6  2006/08/23 04:09:14  steve
+ *  missing sig diagnostics.
+ *
+ * Revision 1.14.2.5  2006/08/15 03:41:24  steve
+ *  Improve performance of unlink of heavily connected nexa.
+ *
+ * Revision 1.14.2.4  2006/08/08 02:17:48  steve
+ *  Improved nexus management performance.
+ *
+ * Revision 1.14.2.3  2006/07/23 19:42:33  steve
+ *  Handle statement output override better in blocks.
+ *
+ * Revision 1.14.2.2  2006/01/27 02:05:46  steve
+ *  Speed up processing of connect when one side is empty.
+ *
+ * Revision 1.14.2.1  2005/09/25 23:30:31  steve
+ *  More predictable ordering of items in NexusSet.
+ *
  * Revision 1.14  2004/02/18 17:11:56  steve
  *  Use perm_strings for named langiage items.
  *
@@ -523,35 +628,5 @@ bool NexusSet::intersect(const NexusSet&that) const
  * Revision 1.7  2002/06/24 01:49:39  steve
  *  Make link_drive_constant cache its results in
  *  the Nexus, to improve cprop performance.
- *
- * Revision 1.6  2002/04/21 04:59:08  steve
- *  Add support for conbinational events by finding
- *  the inputs to expressions and some statements.
- *  Get case and assignment statements working.
- *
- * Revision 1.5  2001/07/25 03:10:49  steve
- *  Create a config.h.in file to hold all the config
- *  junk, and support gcc 3.0. (Stephan Boettcher)
- *
- * Revision 1.4  2000/10/06 23:46:50  steve
- *  ivl_target updates, including more complete
- *  handling of ivl_nexus_t objects. Much reduced
- *  dependencies on pointers to netlist objects.
- *
- * Revision 1.3  2000/08/26 00:54:03  steve
- *  Get at gate information for ivl_target interface.
- *
- * Revision 1.2  2000/07/14 06:12:57  steve
- *  Move inital value handling from NetNet to Nexus
- *  objects. This allows better propogation of inital
- *  values.
- *
- *  Clean up constant propagation  a bit to account
- *  for regs that are not really values.
- *
- * Revision 1.1  2000/06/25 19:59:42  steve
- *  Redesign Links to include the Nexus class that
- *  carries properties of the connected set of links.
- *
  */
 
