@@ -29,7 +29,7 @@
 # include  "netlist.h"
 # include  "netmisc.h"
 
-NexusSet* NetExpr::nex_input()
+NexusSet* NetExpr::nex_input(bool rem_out)
 {
       cerr << get_line()
 	   << ": internal error: nex_input not implemented: "
@@ -37,7 +37,7 @@ NexusSet* NetExpr::nex_input()
       return 0;
 }
 
-NexusSet* NetProc::nex_input()
+NexusSet* NetProc::nex_input(bool rem_out)
 {
       cerr << get_line()
 	   << ": internal error: NetProc::nex_input not implemented"
@@ -45,29 +45,29 @@ NexusSet* NetProc::nex_input()
       return 0;
 }
 
-NexusSet* NetEBinary::nex_input()
+NexusSet* NetEBinary::nex_input(bool rem_out)
 {
-      NexusSet*result = left_->nex_input();
-      NexusSet*tmp = right_->nex_input();
+      NexusSet*result = left_->nex_input(rem_out);
+      NexusSet*tmp = right_->nex_input(rem_out);
       result->add(*tmp);
       delete tmp;
       return result;
 }
 
-NexusSet* NetEBitSel::nex_input()
+NexusSet* NetEBitSel::nex_input(bool rem_out)
 {
-      NexusSet*result = sig_->nex_input();
-      NexusSet*tmp = idx_->nex_input();
+      NexusSet*result = sig_->nex_input(rem_out);
+      NexusSet*tmp = idx_->nex_input(rem_out);
       result->add(*tmp);
       delete tmp;
       return result;
 }
 
-NexusSet* NetEConcat::nex_input()
+NexusSet* NetEConcat::nex_input(bool rem_out)
 {
-      NexusSet*result = parms_[0]->nex_input();
+      NexusSet*result = parms_[0]->nex_input(rem_out);
       for (unsigned idx = 1 ;  idx < parms_.count() ;  idx += 1) {
-	    NexusSet*tmp = parms_[idx]->nex_input();
+	    NexusSet*tmp = parms_[idx]->nex_input(rem_out);
 	    result->add(*tmp);
 	    delete tmp;
       }
@@ -77,19 +77,19 @@ NexusSet* NetEConcat::nex_input()
 /*
  * A constant has not inputs, so always return an empty set.
  */
-NexusSet* NetEConst::nex_input()
+NexusSet* NetEConst::nex_input(bool rem_out)
 {
       return new NexusSet;
 }
 
-NexusSet* NetECReal::nex_input()
+NexusSet* NetECReal::nex_input(bool rem_out)
 {
       return new NexusSet;
 }
 
-NexusSet* NetEMemory::nex_input()
+NexusSet* NetEMemory::nex_input(bool rem_out)
 {
-      NexusSet*result = idx_->nex_input();
+      NexusSet*result = idx_->nex_input(rem_out);
       return result;
 }
 
@@ -97,45 +97,45 @@ NexusSet* NetEMemory::nex_input()
  * A parameter by definition has no inputs. It represents a constant
  * value, even if that value is a constant expression.
  */
-NexusSet* NetEParam::nex_input()
+NexusSet* NetEParam::nex_input(bool rem_out)
 {
       return new NexusSet;
 }
 
-NexusSet* NetEEvent::nex_input()
+NexusSet* NetEEvent::nex_input(bool rem_out)
 {
       return new NexusSet;
 }
 
-NexusSet* NetEScope::nex_input()
+NexusSet* NetEScope::nex_input(bool rem_out)
 {
       return new NexusSet;
 }
 
-NexusSet* NetESelect::nex_input()
+NexusSet* NetESelect::nex_input(bool rem_out)
 {
-      NexusSet*result = base_? base_->nex_input() : new NexusSet();
-      NexusSet*tmp = expr_->nex_input();
+      NexusSet*result = base_? base_->nex_input(rem_out) : new NexusSet();
+      NexusSet*tmp = expr_->nex_input(rem_out);
       result->add(*tmp);
       delete tmp;
       return result;
 }
 
-NexusSet* NetESFunc::nex_input()
+NexusSet* NetESFunc::nex_input(bool rem_out)
 {
       if (nparms_ == 0)
 	    return new NexusSet;
 
-      NexusSet*result = parms_[0]->nex_input();
+      NexusSet*result = parms_[0]->nex_input(rem_out);
       for (unsigned idx = 1 ;  idx < nparms_ ;  idx += 1) {
-	    NexusSet*tmp = parms_[idx]->nex_input();
+	    NexusSet*tmp = parms_[idx]->nex_input(rem_out);
 	    result->add(*tmp);
 	    delete tmp;
       }
       return result;
 }
 
-NexusSet* NetESignal::nex_input()
+NexusSet* NetESignal::nex_input(bool rem_out)
 {
       NexusSet*result = new NexusSet;
       assert(msi_ <= net_->pin_count());
@@ -145,27 +145,27 @@ NexusSet* NetESignal::nex_input()
       return result;
 }
 
-NexusSet* NetETernary::nex_input()
+NexusSet* NetETernary::nex_input(bool rem_out)
 {
       NexusSet*tmp;
-      NexusSet*result = cond_->nex_input();
+      NexusSet*result = cond_->nex_input(rem_out);
 
-      tmp = true_val_->nex_input();
+      tmp = true_val_->nex_input(rem_out);
       result->add(*tmp);
       delete tmp;
 
-      tmp = false_val_->nex_input();
+      tmp = false_val_->nex_input(rem_out);
       result->add(*tmp);
       delete tmp;
 
       return result;
 }
 
-NexusSet* NetEUFunc::nex_input()
+NexusSet* NetEUFunc::nex_input(bool rem_out)
 {
       NexusSet*result = new NexusSet;
       for (unsigned idx = 0 ;  idx < parms_.count() ;  idx += 1) {
-	    NexusSet*tmp = parms_[idx]->nex_input();
+	    NexusSet*tmp = parms_[idx]->nex_input(rem_out);
 	    result->add(*tmp);
 	    delete tmp;
       }
@@ -173,33 +173,33 @@ NexusSet* NetEUFunc::nex_input()
       return result;
 }
 
-NexusSet* NetEUnary::nex_input()
+NexusSet* NetEUnary::nex_input(bool rem_out)
 {
-      return expr_->nex_input();
+      return expr_->nex_input(rem_out);
 }
 
-NexusSet* NetEVariable::nex_input()
+NexusSet* NetEVariable::nex_input(bool rem_out)
 {
       return new NexusSet;
 }
 
-NexusSet* NetAssign_::nex_input()
+NexusSet* NetAssign_::nex_input(bool rem_out)
 {
       if (bmux_ == 0)
 	    return new NexusSet;
       else
-	    return bmux_->nex_input();
+	    return bmux_->nex_input(rem_out);
 }
 
-NexusSet* NetAssignBase::nex_input()
+NexusSet* NetAssignBase::nex_input(bool rem_out)
 {
-      NexusSet*result = rval_->nex_input();
+      NexusSet*result = rval_->nex_input(rem_out);
 
 	/* It is possible that the lval_ can hav nex_input values. In
 	   particular, index expressions are statement inputs as well,
 	   so should be addressed here. */
       for (NetAssign_*cur = lval_ ;  cur ;  cur = cur->more) {
-	    NexusSet*tmp = cur->nex_input();
+	    NexusSet*tmp = cur->nex_input(rem_out);
 	    result->add(*tmp);
 	    delete tmp;
       }
@@ -224,7 +224,7 @@ NexusSet* NetAssignBase::nex_input()
  * In this example, "t" should not be in the input set because it is
  * used by the sequence as a temporary value.
  */
-NexusSet* NetBlock::nex_input()
+NexusSet* NetBlock::nex_input(bool rem_out)
 {
       if (last_ == 0)
 	    return new NexusSet;
@@ -244,24 +244,24 @@ NexusSet* NetBlock::nex_input()
 
       do {
 	      /* Get the inputs for the current statement. */
-	    NexusSet*tmp = cur->nex_input();
+	    NexusSet*tmp = cur->nex_input(rem_out);
 
-	      /* Remove from the input set those bits that are outputs
-		 from previous statements. They aren't really inputs
-		 to the block, just internal intermediate values. */
-	    tmp->rem(*prev);
-
-	      /* Add the corrected set to the accumulated input set. */
+	      /* Add the current input set to the accumulated input set. */
 	    result->add(*tmp);
 	    delete tmp;
 
-	      /* Add the current outputs to the accumulated output
-		 set, for processing of later statements. */
+	      /* Add the current outputs to the accumulated output set,
+		 so they can be removed from the input set below. */
 	    cur->nex_output(*prev);
 
 	    cur = cur->next_;
       } while (cur != last_->next_);
 
+        /* Remove from the input set those bits that are outputs
+           from other statements. They aren't really inputs
+           to the block, just internal intermediate values. */
+      if (rem_out) result->rem(*prev);
+
       return result;
 }
 
@@ -270,9 +270,9 @@ NexusSet* NetBlock::nex_input()
  * the inputs to all the guards, and the inputs to all the guarded
  * statements.
  */
-NexusSet* NetCase::nex_input()
+NexusSet* NetCase::nex_input(bool rem_out)
 {
-      NexusSet*result = expr_->nex_input();
+      NexusSet*result = expr_->nex_input(rem_out);
       if (result == 0)
 	    return 0;
 
@@ -282,7 +282,7 @@ NexusSet* NetCase::nex_input()
 	    if (items_[idx].statement == 0)
 		  continue;
 
-	    NexusSet*tmp = items_[idx].statement->nex_input();
+	    NexusSet*tmp = items_[idx].statement->nex_input(rem_out);
 	    assert(tmp);
 	    result->add(*tmp);
 	    delete tmp;
@@ -291,7 +291,7 @@ NexusSet* NetCase::nex_input()
 		 case is special and is identified by a null
 		 guard. The default guard obviously has no input. */
 	    if (items_[idx].guard) {
-		  tmp = items_[idx].guard->nex_input();
+		  tmp = items_[idx].guard->nex_input(rem_out);
 		  assert(tmp);
 		  result->add(*tmp);
 		  delete tmp;
@@ -301,24 +301,24 @@ NexusSet* NetCase::nex_input()
       return result;
 }
 
-NexusSet* NetCAssign::nex_input()
+NexusSet* NetCAssign::nex_input(bool rem_out)
 {
       cerr << get_line() << ": internal warning: NetCAssign::nex_input()"
 	   << " not implemented." << endl;
       return new NexusSet;
 }
 
-NexusSet* NetCondit::nex_input()
+NexusSet* NetCondit::nex_input(bool rem_out)
 {
-      NexusSet*result = expr_->nex_input();
+      NexusSet*result = expr_->nex_input(rem_out);
       if (if_ != 0) {
-	    NexusSet*tmp = if_->nex_input();
+	    NexusSet*tmp = if_->nex_input(rem_out);
 	    result->add(*tmp);
 	    delete tmp;
       }
 
       if (else_ != 0) {
-	    NexusSet*tmp = else_->nex_input();
+	    NexusSet*tmp = else_->nex_input(rem_out);
 	    result->add(*tmp);
 	    delete tmp;
       }
@@ -326,16 +326,16 @@ NexusSet* NetCondit::nex_input()
       return result;
 }
 
-NexusSet* NetForce::nex_input()
+NexusSet* NetForce::nex_input(bool rem_out)
 {
       cerr << get_line() << ": internal warning: NetForce::nex_input()"
 	   << " not implemented." << endl;
       return new NexusSet;
 }
 
-NexusSet* NetForever::nex_input()
+NexusSet* NetForever::nex_input(bool rem_out)
 {
-      NexusSet*result = statement_->nex_input();
+      NexusSet*result = statement_->nex_input(rem_out);
       return result;
 }
 
@@ -348,29 +348,29 @@ NexusSet* NetForever::nex_input()
  * include the input set of the <expr> because it does not affect the
  * result.
  */
-NexusSet* NetPDelay::nex_input()
+NexusSet* NetPDelay::nex_input(bool rem_out)
 {
-      NexusSet*result = statement_->nex_input();
+      NexusSet*result = statement_->nex_input(rem_out);
       return result;
 }
 
-NexusSet* NetRepeat::nex_input()
+NexusSet* NetRepeat::nex_input(bool rem_out)
 {
-      NexusSet*result = statement_->nex_input();
-      NexusSet*tmp = expr_->nex_input();
+      NexusSet*result = statement_->nex_input(rem_out);
+      NexusSet*tmp = expr_->nex_input(rem_out);
       result->add(*tmp);
       delete tmp;
       return result;
 }
 
-NexusSet* NetSTask::nex_input()
+NexusSet* NetSTask::nex_input(bool rem_out)
 {
       if (parms_.count() == 0)
 	    return new NexusSet;
 
-      NexusSet*result = parms_[0]->nex_input();
+      NexusSet*result = parms_[0]->nex_input(rem_out);
       for (unsigned idx = 1 ;  idx < parms_.count() ;  idx += 1) {
-	    NexusSet*tmp = parms_[idx]->nex_input();
+	    NexusSet*tmp = parms_[idx]->nex_input(rem_out);
 	    result->add(*tmp);
 	    delete tmp;
       }
@@ -383,15 +383,15 @@ NexusSet* NetSTask::nex_input()
  * parameters to consider, because the compiler already removed them
  * and converted them to blocking assignments.
  */
-NexusSet* NetUTask::nex_input()
+NexusSet* NetUTask::nex_input(bool rem_out)
 {
       return new NexusSet;
 }
 
-NexusSet* NetWhile::nex_input()
+NexusSet* NetWhile::nex_input(bool rem_out)
 {
-      NexusSet*result = proc_->nex_input();
-      NexusSet*tmp = cond_->nex_input();
+      NexusSet*result = proc_->nex_input(rem_out);
+      NexusSet*tmp = cond_->nex_input(rem_out);
       result->add(*tmp);
       delete tmp;
       return result;
