@@ -17,7 +17,7 @@
  *    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
  */
 #ifdef HAVE_CVS_IDENT
-#ident "$Id: netlist.cc,v 1.226 2004/10/04 01:10:54 steve Exp $"
+#ident "$Id: netlist.cc,v 1.226.2.7 2006/04/23 04:26:14 steve Exp $"
 #endif
 
 # include "config.h"
@@ -229,7 +229,7 @@ NetNode::~NetNode()
 NetNet::NetNet(NetScope*s, perm_string n, Type t, unsigned npins)
 : NetObj(s, n, npins), sig_next_(0), sig_prev_(0),
     type_(t), port_type_(NOT_A_PORT), signed_(false), msb_(npins-1), lsb_(0),
-    local_flag_(false), eref_count_(0), lref_count_(0)
+    local_flag_(false), eref_count_(0), lref_count_(0), mref_(0)
 {
       assert(s);
 
@@ -270,7 +270,7 @@ NetNet::NetNet(NetScope*s, perm_string n, Type t, long ms, long ls)
 : NetObj(s, n, ((ms>ls)?ms-ls:ls-ms) + 1),
     sig_next_(0), sig_prev_(0), type_(t),
     port_type_(NOT_A_PORT), signed_(false), msb_(ms), lsb_(ls),
-    local_flag_(false), eref_count_(0), lref_count_(0)
+    local_flag_(false), eref_count_(0), lref_count_(0), mref_(0)
 {
       assert(s);
 
@@ -322,6 +322,14 @@ NetNet::~NetNet()
 	    dump_net(cerr, 4);
       }
       assert(lref_count_ == 0);
+      if (mref_ != 0) {
+	    cerr << get_line() << ": internal error: attempt to delete "
+		 << "signal ``" << name() << "'' which has "
+		 << "memory explode references." << endl;
+	    dump_net(cerr, 4);
+      }
+      assert(mref_ == 0);
+
       if (scope())
 	    scope()->rem_signal(this);
 
@@ -462,6 +470,23 @@ unsigned NetNet::get_refs() const
       return lref_count_ + eref_count_;
 }
 
+void NetNet::mref(NetMemory*mem)
+{
+      assert(mref_ == 0);
+      assert(mem != 0);
+      mref_ = mem;
+}
+
+NetMemory*NetNet::mref()
+{
+      return mref_;
+}
+
+const NetMemory* NetNet::mref() const
+{
+      return mref_;
+}
+
 
 NetSubnet::NetSubnet(NetNet*sig, unsigned off, unsigned wid)
 : NetNet(sig->scope(), sig->scope()->local_symbol(), sig->type(), wid)
@@ -533,6 +558,8 @@ const NetScope* NetProcTop::scope() const
 NetFF::NetFF(NetScope*s, perm_string n, unsigned wid)
 : NetNode(s, n, 8 + 2*wid)
 {
+      demux_ = 0;
+
       pin_Clock().set_dir(Link::INPUT);
       pin_Clock().set_name(perm_string::literal("Clock"), 0);
       pin_Enable().set_dir(Link::INPUT);
@@ -684,6 +711,118 @@ const verinum& NetFF::sset_value() const
       return sset_value_;
 }
 
+unsigned NetDecode::width() const
+{
+      return width_;
+}
+
+unsigned NetDecode::awidth() const
+{
+      return pin_count();
+}
+
+NetDemux::NetDemux(NetScope*s, perm_string name,
+		   unsigned bus_width, unsigned address_width, unsigned size)
+: NetNode(s, name, bus_width*2+address_width+bus_width/size)
+{
+      width_ = bus_width;
+      awidth_ = address_width;
+      size_ = size;
+
+      for (unsigned idx = 0 ;  idx < width_ ;  idx += 1) {
+	    pin_Q(idx).set_dir(Link::OUTPUT);
+	    pin_Q(idx).set_name(perm_string::literal("Q"), idx);
+      }
+      for (unsigned idx = 0 ;  idx < width_ ;  idx += 1) {
+	    pin_Data(idx).set_dir(Link::INPUT);
+	    pin_Data(idx).set_name(perm_string::literal("Data"), idx);
+      }
+      for (unsigned idx = 0 ;  idx < awidth_ ;  idx += 1) {
+	    pin_Address(idx).set_dir(Link::INPUT);
+	    pin_Address(idx).set_name(perm_string::literal("Address"), idx);
+      }
+      for (unsigned idx = 0 ;  idx < width_/size_ ;  idx += 1) {
+	    pin_WriteData(idx).set_dir(Link::INPUT);
+	    pin_WriteData(idx).set_name(perm_string::literal("Writedata"), idx);
+      }
+}
+
+NetDemux::~NetDemux()
+{
+}
+
+unsigned NetDemux::width() const
+{
+      return width_;
+}
+
+unsigned NetDemux::awidth() const
+{
+      return awidth_;
+}
+
+unsigned NetDemux::size() const
+{
+      return size_;
+}
+
+Link& NetDemux::pin_Q(unsigned idx)
+{
+      assert(idx < width_);
+      return pin(idx);
+}
+
+const Link& NetDemux::pin_Q(unsigned idx) const
+{
+      assert(idx < width_);
+      return pin(idx);
+}
+
+Link& NetDemux::pin_Data(unsigned idx)
+{
+      assert(idx < width_);
+      return pin(width_+idx);
+}
+
+const Link& NetDemux::pin_Data(unsigned idx) const
+{
+      assert(idx < width_);
+      return pin(width_+idx);
+}
+
+Link& NetDemux::pin_Address(unsigned idx)
+{
+      assert(idx < awidth_);
+      return pin(width_+width_+idx);
+}
+
+const Link& NetDemux::pin_Address(unsigned idx) const
+{
+      assert(idx < awidth_);
+      return pin(width_+width_+idx);
+}
+
+Link& NetDemux::pin_WriteData(unsigned idx)
+{
+      assert(idx < width_/size_);
+      return pin(width_+width_+awidth_+idx);
+}
+
+const Link& NetDemux::pin_WriteData(unsigned idx) const
+{
+      assert(idx < width_/size_);
+      return pin(width_+width_+awidth_+idx);
+}
+
+NetDecode* NetFF::get_demux()
+{
+      return demux_;
+}
+
+const NetDecode* NetFF::get_demux() const
+{
+      return demux_;
+}
 
 /*
  * The NetAddSub class represents an LPM_ADD_SUB device. The pinout is
@@ -1060,6 +1199,40 @@ const Link& NetCompare::pin_DataB(unsigned idx) const
       return pin(8+width_+idx);
 }
 
+NetDecode::NetDecode(NetScope*s, perm_string name, NetFF*mem,
+		     unsigned awid, unsigned word_width)
+: NetNode(s, name, awid)
+{
+      width_ = word_width;
+      assert( mem->width() % width_ == 0 );
+
+      ff_ = mem;
+      ff_->demux_ = this;
+      make_pins_(awid);
+}
+
+NetDecode::~NetDecode()
+{
+}
+
+void NetDecode::make_pins_(unsigned awid)
+{
+      for (unsigned idx = 0 ;  idx < awid ;  idx += 1) {
+	    pin(idx).set_dir(Link::INPUT);
+	    pin(idx).set_name(perm_string::literal("Address"), idx);
+      }
+}
+
+Link& NetDecode::pin_Address(unsigned idx)
+{
+      return pin(idx);
+}
+
+const Link& NetDecode::pin_Address(unsigned idx) const
+{
+      return pin(idx);
+}
+
 NetDivide::NetDivide(NetScope*sc, perm_string n, unsigned wr,
 		     unsigned wa, unsigned wb)
 : NetNode(sc, n, wr+wa+wb),
@@ -1385,10 +1558,7 @@ const Link& NetMux::pin_Data(unsigned w, unsigned s) const
       return pin(2+width_+swidth_+s*width_+w);
 }
 
-
-NetRamDq::NetRamDq(NetScope*s, perm_string n, NetMemory*mem, unsigned awid)
-: NetNode(s, n, 3+2*mem->width()+awid),
-  mem_(mem), awidth_(awid)
+void NetRamDq::make_pins_(unsigned wid)
 {
       pin(0).set_dir(Link::INPUT); pin(0).set_name(perm_string::literal("InClock"), 0);
       pin(1).set_dir(Link::INPUT); pin(1).set_name(perm_string::literal("OutClock"), 0);
@@ -1399,39 +1569,49 @@ NetRamDq::NetRamDq(NetScope*s, perm_string n, NetMemory*mem, unsigned awid)
 	    pin(3+idx).set_name(perm_string::literal("Address"), idx);
       }
 
-      for (unsigned idx = 0 ;  idx < width() ;  idx += 1) {
+      for (unsigned idx = 0 ;  idx < wid ;  idx += 1) {
 	    pin(3+awidth_+idx).set_dir(Link::INPUT);
 	    pin(3+awidth_+idx).set_name(perm_string::literal("Data"), idx);
       }
 
-      for (unsigned idx = 0 ;  idx < width() ;  idx += 1) {
-	    pin(3+awidth_+width()+idx).set_dir(Link::OUTPUT);
-	    pin(3+awidth_+width()+idx).set_name(perm_string::literal("Q"), idx);
+      for (unsigned idx = 0 ;  idx < wid ;  idx += 1) {
+	    pin(3+awidth_+wid+idx).set_dir(Link::OUTPUT);
+	    pin(3+awidth_+wid+idx).set_name(perm_string::literal("Q"), idx);
       }
+}
 
+
+NetRamDq::NetRamDq(NetScope*s, perm_string n, NetMemory*mem, unsigned awid)
+: NetNode(s, n, 3+2*mem->width()+awid),
+  mem_(mem), awidth_(awid)
+{
+      make_pins_(mem->width());
       next_ = mem_->ram_list_;
       mem_->ram_list_ = this;
 }
 
 NetRamDq::~NetRamDq()
 {
-      if (mem_->ram_list_ == this) {
-	    mem_->ram_list_ = next_;
+      if (mem_) {
+	    if (mem_->ram_list_ == this) {
+		  mem_->ram_list_ = next_;
 
-      } else {
-	    NetRamDq*cur = mem_->ram_list_;
-	    while (cur->next_ != this) {
-		  assert(cur->next_);
-		  cur = cur->next_;
+	    } else {
+		  NetRamDq*cur = mem_->ram_list_;
+		  while (cur->next_ != this) {
+			assert(cur->next_);
+			cur = cur->next_;
+		  }
+		  assert(cur->next_ == this);
+		  cur->next_ = next_;
 	    }
-	    assert(cur->next_ == this);
-	    cur->next_ = next_;
       }
 }
 
 unsigned NetRamDq::width() const
 {
-      return mem_->width();
+      if (mem_) return mem_->width();
+      return 0;
 }
 
 unsigned NetRamDq::awidth() const
@@ -1441,7 +1621,13 @@ unsigned NetRamDq::awidth() const
 
 unsigned NetRamDq::size() const
 {
-      return mem_->count();
+      if (mem_) return mem_->count();
+      return 0;
+}
+
+NetMemory* NetRamDq::mem()
+{
+      return mem_;
 }
 
 const NetMemory* NetRamDq::mem() const
@@ -2003,6 +2189,7 @@ NetMemory::NetMemory(NetScope*sc, perm_string n, long w, long s, long e)
 : width_(w), idxh_(s), idxl_(e), ram_list_(0), scope_(sc)
 {
       name_ = n;
+      explode_ = 0;
       scope_->add_memory(this);
 }
 
@@ -2033,7 +2220,25 @@ unsigned NetMemory::index_to_address(long idx) const
 	    return idx - idxl_;
 }
 
+NetNet* NetMemory::explode_to_reg()
+{
+      if (explode_)
+	    return explode_;
 
+      explode_ = new NetNet(scope_, name_, NetNet::REG, count()*width_);
+	//explode_->incr_lref();
+      return explode_;
+}
+
+NetNet* NetMemory::reg_from_explode()
+{
+      return explode_;
+}
+
+const NetNet* NetMemory::reg_from_explode() const
+{
+      return explode_;
+}
 
 NetEMemory* NetEMemory::dup_expr() const
 {
@@ -2281,6 +2486,27 @@ const NetProc*NetTaskDef::proc() const
 
 /*
  * $Log: netlist.cc,v $
+ * Revision 1.226.2.7  2006/04/23 04:26:14  steve
+ *  Constant propagate addresses through NetRamDq read ports.
+ *
+ * Revision 1.226.2.6  2006/04/16 19:26:38  steve
+ *  Fix handling of exploded memories with partial or missing resets.
+ *
+ * Revision 1.226.2.5  2006/03/26 23:09:22  steve
+ *  Handle asynchronous demux/bit replacements.
+ *
+ * Revision 1.226.2.4  2006/03/16 05:40:18  steve
+ *  Fix crash when memory exploding doesnot work
+ *
+ * Revision 1.226.2.3  2006/03/12 07:34:17  steve
+ *  Fix the memsynth1 case.
+ *
+ * Revision 1.226.2.2  2006/02/19 00:11:32  steve
+ *  Handle synthesis of FF vectors with l-value decoder.
+ *
+ * Revision 1.226.2.1  2006/01/18 01:23:23  steve
+ *  Rework l-value handling to allow for more l-value type flexibility.
+ *
  * Revision 1.226  2004/10/04 01:10:54  steve
  *  Clean up spurious trailing white space.
  *
