@@ -1753,6 +1753,9 @@ void dll_target::lpm_latch( const NetLatch *latchPtr )
       objPtr->attr = fill_in_attributes( latchPtr );
 
       scope_add_lpm( objPtr->scope, objPtr );
+
+      // Set the gate signal to point to the nexus, and the nexus to point back to this device.
+      const Nexus *const nexPtr = latchPtr->pin_Gate().nexus();
     }
   catch ( bad_alloc &memoryAllocationException )
     {
