@@ -238,8 +238,8 @@ bool target_xnf::start_design(const Design*des)
 	    ncf_.open(ncfpath.c_str());
 
       out_ << "LCANET,6" << endl;
-      out_ << "PROG,verilog,$Name: v0_8 $,\"Icarus Verilog\"" << endl;
-      ncf_ << "# Generated by Icarus Verilog $Name: v0_8 $" << endl;
+      out_ << "PROG,verilog,$Name:  $,\"Icarus Verilog\"" << endl;
+      ncf_ << "# Generated by Icarus Verilog $Name:  $" << endl;
 
       if (des->get_flag("part") != 0) {
 	    out_ << "PART," << des->get_flag("part") << endl;
