@@ -17,7 +17,7 @@
  *    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
  */
 #ifdef HAVE_CVS_IDENT
-#ident "$Id: target.cc,v 1.69 2004/05/31 23:34:39 steve Exp $"
+#ident "$Id: target.cc,v 1.69.2.2 2006/03/26 23:09:25 steve Exp $"
 #endif
 
 # include "config.h"
@@ -101,6 +101,20 @@ void target_t::lpm_compare(const NetCompare*)
 	    "Unhandled NetCompare." << endl;
 }
 
+bool target_t::lpm_decode(const NetDecode*)
+{
+      cerr << "target (" << typeid(*this).name() << "): "
+	    "Unhandled NetDecode." << endl;
+      return false;
+}
+
+bool target_t::lpm_demux(const NetDemux*)
+{
+      cerr << "target (" << typeid(*this).name() << "): "
+	    "Unhandled NetDemux." << endl;
+      return false;
+}
+
 void target_t::lpm_divide(const NetDivide*)
 {
       cerr << "target (" << typeid(*this).name() << "): "
@@ -119,6 +133,11 @@ void target_t::lpm_ff(const NetFF*)
 	    "Unhandled NetFF." << endl;
 }
 
+void target_t::lpm_latch( const NetLatch * )
+{
+      cerr << "target (" << typeid( *this ).name() << "): Unhandled NetLatch." << endl;
+}
+
 void target_t::lpm_mult(const NetMult*)
 {
       cerr << "target (" << typeid(*this).name() << "): "
@@ -420,6 +439,12 @@ void expr_scan_t::expr_binary(const NetEBinary*ex)
 
 /*
  * $Log: target.cc,v $
+ * Revision 1.69.2.2  2006/03/26 23:09:25  steve
+ *  Handle asynchronous demux/bit replacements.
+ *
+ * Revision 1.69.2.1  2006/02/19 00:11:34  steve
+ *  Handle synthesis of FF vectors with l-value decoder.
+ *
  * Revision 1.69  2004/05/31 23:34:39  steve
  *  Rewire/generalize parsing an elaboration of
  *  function return values to allow for better
