@@ -17,7 +17,7 @@
  *    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
  */
 #ifdef HAVE_CVS_IDENT
-#ident "$Id: stub.c,v 1.90 2004/10/04 01:10:57 steve Exp $"
+#ident "$Id: stub.c,v 1.90.2.11 2006/09/15 23:56:05 steve Exp $"
 #endif
 
 # include "config.h"
@@ -35,6 +35,23 @@
 
 static FILE*out;
 
+static void show_attribute(ivl_attribute_t attr, unsigned ind)
+{
+      switch (attr->type) {
+	  case IVL_ATT_VOID:
+	    fprintf(out, "%*s(* %s *)\n", ind,"", attr->key);
+	    break;
+	  case IVL_ATT_STR:
+	    fprintf(out, "%*s(* %s = \"%s\" *)\n", ind,"", attr->key,
+		    attr->val.str);
+	    break;
+	  case IVL_ATT_NUM:
+	    fprintf(out, "%*s(* %s = %ld *)\n", ind,"", attr->key,
+		    attr->val.num);
+	    break;
+      }
+}
+
 static void show_expression(ivl_expr_t net, unsigned ind)
 {
       unsigned idx;
@@ -64,12 +81,21 @@ static void show_expression(ivl_expr_t net, unsigned ind)
 	    show_expression(ivl_expr_oper1(net), ind+3);
 	    break;
 
-	  case IVL_EX_BINARY:
-	    fprintf(out, "%*s<\"%c\" width=%u, %s, type=%s>\n", ind, "",
-		    ivl_expr_opcode(net), width, sign, vt);
-	    show_expression(ivl_expr_oper1(net), ind+3);
-	    show_expression(ivl_expr_oper2(net), ind+3);
-	    break;
+	  case IVL_EX_BINARY: {
+		ivl_expr_t oper1 = ivl_expr_oper1(net);
+		ivl_expr_t oper2 = ivl_expr_oper2(net);
+		fprintf(out, "%*s<\"%c\" width=%u, %s, type=%s>\n", ind, "",
+			ivl_expr_opcode(net), width, sign, vt);
+		if (oper1)
+		      show_expression(oper1, ind+3);
+		else
+		      fprintf(out, "%*sERROR: nil oper1\n", ind+3, "");
+		if (oper2)
+		      show_expression(oper2, ind+3);
+		else
+		      fprintf(out, "%*sERROR: nil oper2\n", ind+3, "");
+		break;
+	  }
 
 	  case IVL_EX_CONCAT:
 	    fprintf(out, "%*s<concat repeat=%u, width=%u, %s, type=%s>\n",
@@ -95,13 +121,23 @@ static void show_expression(ivl_expr_t net, unsigned ind)
 		break;
 	  }
 
+	  case IVL_EX_ULONG: {
+		fprintf(out, "%*s<ulong=%u'd%lu, %s>\n",
+			ind, "", width, ivl_expr_uvalue(net), sign);
+		break;
+	  }
+
 	  case IVL_EX_SELECT:
 	      /* The SELECT expression can be used to express part
 		 select, or if the base is null vector extension. */
 	    if (ivl_expr_oper2(net)) {
+		  ivl_expr_t o1 = ivl_expr_oper1(net);
 		  fprintf(out, "%*s<select: width=%u, %s>\n", ind, "",
 			  width, sign);
-		  show_expression(ivl_expr_oper1(net), ind+3);
+		  if (o1)
+			show_expression(ivl_expr_oper1(net), ind+3);
+		  else
+			fprintf(out, "%*sERROR: Missing oper1\n", ind+3, "");
 		  show_expression(ivl_expr_oper2(net), ind+3);
 	    } else {
 		  fprintf(out, "%*s<expr pad: width=%u, %s>\n", ind, "",
@@ -181,6 +217,72 @@ static void show_expression(ivl_expr_t net, unsigned ind)
       }
 }
 
+static void show_lpm_ram(ivl_lpm_t net)
+{
+      unsigned width = ivl_lpm_width(net);
+      unsigned count = ivl_lpm_size(net);
+
+      unsigned idx, sdx;
+
+      fprintf(out, "  LPM_RAM_DQ %s (word-width=%u, count=%u)\n",
+	      ivl_lpm_basename(net), ivl_lpm_width(net), count);
+      for (idx = 0 ;  idx < ivl_lpm_selects(net) ;  idx += 1) {
+	    ivl_nexus_t nex = ivl_lpm_select(net, idx);
+	    fprintf(out, "    Address %u: %s\n", idx,
+		    nex? ivl_nexus_name(nex) : "");
+      }
+
+      for (idx = 0 ;  idx < width ;  idx += 1) {
+	    ivl_nexus_t nex = ivl_lpm_q(net, idx);
+	    fprintf(out, "    Q %u: %s\n", idx, nex? ivl_nexus_name(nex) : "");
+      }
+
+      for (sdx = 0 ;  sdx < count ;  sdx += 1) {
+	    for (idx = 0 ;  idx < width ;  idx += 1) {
+		  ivl_nexus_t nex = ivl_lpm_data2(net, sdx, idx);
+		  fprintf(out, "    Word%u %u: %s\n",
+			  sdx, idx, nex? ivl_nexus_name(nex) : "");
+	    }
+      }
+}
+
+static void show_lpm_demux(ivl_lpm_t net)
+{
+      unsigned idx;
+      unsigned width = ivl_lpm_width(net);
+      unsigned awid = ivl_lpm_selects(net);
+      unsigned size = ivl_lpm_size(net);
+
+      fprintf(out, "  LPM_DEMUX %s (bus-width=%u, addr-width=%u, size=%u)\n",
+	      ivl_lpm_basename(net), width, awid, size);
+
+      if (width%size != 0) {
+	    fprintf(out, "    ERROR: width %% size == %u\n", width%size);
+      }
+
+      for (idx = 0 ;  idx < width ;  idx += 1) {
+	    ivl_nexus_t nex = ivl_lpm_q(net, idx);
+	    fprintf(out, "    Q %u: %s\n", idx,
+		    nex? ivl_nexus_name(nex) : "");
+      }
+      for (idx = 0 ;  idx < width ;  idx += 1) {
+	    ivl_nexus_t nex = ivl_lpm_data(net, idx);
+	    fprintf(out, "    Data %u: %s\n", idx,
+		    nex? ivl_nexus_name(nex) : "");
+      }
+      for (idx = 0 ;  idx < awid ;  idx += 1) {
+	    ivl_nexus_t nex = ivl_lpm_select(net, idx);
+	    fprintf(out, "    Address %u: %s\n", idx,
+		    nex? ivl_nexus_name(nex) : "");
+      }
+
+      for (idx = 0 ;  idx < width/size ;  idx += 1) {
+	    ivl_nexus_t nex = ivl_lpm_datab(net, idx);
+	    fprintf(out, "    WriteIn %u: %s\n", idx,
+		    nex? ivl_nexus_name(nex) : "");
+      }
+}
+
 static void show_lpm(ivl_lpm_t net)
 {
       unsigned idx;
@@ -227,6 +329,23 @@ static void show_lpm(ivl_lpm_t net)
 		break;
 	  }
 
+	  case IVL_LPM_CMP_EQ: {
+		fprintf(out, "  LPM_COMPARE(EQ) %s: <width=%u>\n",
+			ivl_lpm_basename(net), width);
+		fprintf(out, "    Q: %s\n", ivl_nexus_name(ivl_lpm_q(net, 0)));
+		for (idx = 0 ;  idx < width ;  idx += 1) {
+		      ivl_nexus_t nex = ivl_lpm_data(net, idx);
+		      fprintf(out, "    Data A %u: %s\n", idx,
+			      nex? ivl_nexus_name(nex) : "");
+		}
+		for (idx = 0 ;  idx < width ;  idx += 1) {
+		      ivl_nexus_t nex = ivl_lpm_datab(net, idx);
+		      fprintf(out, "    Data B %u: %s\n", idx,
+			      nex? ivl_nexus_name(nex) : "");
+		}
+		break;
+	  }
+
 	  case IVL_LPM_CMP_NE: {
 		fprintf(out, "  LPM_COMPARE(NE) %s: <width=%u>\n",
 			ivl_lpm_basename(net), width);
@@ -244,6 +363,42 @@ static void show_lpm(ivl_lpm_t net)
 		break;
 	  }
 
+	  case IVL_LPM_CMP_GE: {
+		fprintf(out, "  LPM_COMPARE(GE) %s: <width=%u>\n",
+			ivl_lpm_basename(net), width);
+		fprintf(out, "    Q: %s\n", ivl_nexus_name(ivl_lpm_q(net, 0)));
+		for (idx = 0 ;  idx < width ;  idx += 1) {
+		      ivl_nexus_t nex = ivl_lpm_data(net, idx);
+		      fprintf(out, "    Data A %u: %s\n", idx,
+			      nex? ivl_nexus_name(nex) : "");
+		}
+		for (idx = 0 ;  idx < width ;  idx += 1) {
+		      ivl_nexus_t nex = ivl_lpm_datab(net, idx);
+		      fprintf(out, "    Data B %u: %s\n", idx,
+			      nex? ivl_nexus_name(nex) : "");
+		}
+		break;
+	  }
+
+	  case IVL_LPM_DECODE: {
+		fprintf(out, "  LPM_DECODE %s (word-width=%u)\n",
+			ivl_lpm_basename(net), ivl_lpm_width(net));
+		for (idx = 0 ;  idx < ivl_lpm_selects(net) ;  idx += 1) {
+		      ivl_nexus_t nex = ivl_lpm_select(net, idx);
+		      fprintf(out, "    Address %u: %s\n", idx,
+			      nex? ivl_nexus_name(nex) : "");
+		}
+		break;
+	  }
+
+	  case IVL_LPM_DEMUX:
+	    show_lpm_demux(net);
+	    break;
+
+	  case IVL_LPM_RAM:
+	    show_lpm_ram(net);
+	    break;
+
 	  case IVL_LPM_SHIFTL: {
 		fprintf(out, "  LPM_SHIFTL %s: <width=%u, selects=%u %s>\n",
 			ivl_lpm_basename(net), width, ivl_lpm_selects(net),
@@ -296,6 +451,12 @@ static void show_lpm(ivl_lpm_t net)
 		fprintf(out, "  LPM_FF %s: <width=%u>\n",
 			ivl_lpm_basename(net), width);
 
+		if (ivl_lpm_decode(net)) {
+		      ivl_lpm_t dec = ivl_lpm_decode(net);
+		      fprintf(out, "    decoder: %s\n",
+			      ivl_lpm_basename(dec));
+		}
+
 		if (ivl_lpm_enable(net))
 		      fprintf(out, "    clk: %s CE: %s\n",
 			      ivl_nexus_name(ivl_lpm_clk(net)),
@@ -308,6 +469,10 @@ static void show_lpm(ivl_lpm_t net)
 		      fprintf(out, "    Aclr: %s\n",
 			      ivl_nexus_name(ivl_lpm_async_clr(net)));
 
+		if (ivl_lpm_sync_clr(net))
+		      fprintf(out, "    Sclr: %s\n",
+			      ivl_nexus_name(ivl_lpm_sync_clr(net)));
+
 		if (ivl_lpm_async_set(net)) {
 		      fprintf(out, "    Aset: %s\n",
 			      ivl_nexus_name(ivl_lpm_async_set(net)));
@@ -315,6 +480,13 @@ static void show_lpm(ivl_lpm_t net)
 			    show_expression(ivl_lpm_aset_value(net), 10);
 		}
 
+		if (ivl_lpm_sync_set(net)) {
+		      fprintf(out, "    Sset: %s\n",
+			      ivl_nexus_name(ivl_lpm_sync_set(net)));
+		      if (ivl_lpm_sset_value(net))
+			    show_expression(ivl_lpm_sset_value(net), 10);
+		}
+
 		for (idx = 0 ;  idx < width ;  idx += 1)
 		      fprintf(out, "    Data %u: %s\n", idx,
 			      ivl_nexus_name(ivl_lpm_data(net, idx)));
@@ -352,9 +524,11 @@ static void show_lpm(ivl_lpm_t net)
 			ivl_lpm_basename(net), width, ivl_lpm_size(net),
 			ivl_lpm_selects(net));
 
-		for (idx = 0 ;  idx < width ;  idx += 1)
+		for (idx = 0 ;  idx < width ;  idx += 1) {
+		      ivl_nexus_t nex = ivl_lpm_q(net, idx);
 		      fprintf(out, "    Q %u: %s\n", idx,
-			      ivl_nexus_name(ivl_lpm_q(net, idx)));
+			      nex? ivl_nexus_name(nex) : "");
+		}
 
 		for (idx = 0 ;  idx < ivl_lpm_selects(net) ;  idx += 1)
 		      fprintf(out, "    S %u: %s\n", idx,
@@ -374,6 +548,11 @@ static void show_lpm(ivl_lpm_t net)
 		    ivl_lpm_width(net),
 		    ivl_lpm_signed(net));
       }
+
+      for (idx = 0 ;  idx < ivl_lpm_attr_cnt(net) ;  idx += 1) {
+	    ivl_attribute_t atr = ivl_lpm_attr_val(net,idx);
+	    show_attribute(atr, 4);
+      }
 }
 
 static void show_assign_lval(ivl_lval_t lval, unsigned ind)
@@ -584,19 +763,7 @@ static int show_process(ivl_process_t net, void*x)
 
       for (idx = 0 ;  idx < ivl_process_attr_cnt(net) ;  idx += 1) {
 	    ivl_attribute_t attr = ivl_process_attr_val(net, idx);
-	    switch (attr->type) {
-		case IVL_ATT_VOID:
-		  fprintf(out, "    (* %s *)\n", attr->key);
-		  break;
-		case IVL_ATT_STR:
-		  fprintf(out, "    (* %s = \"%s\" *)\n", attr->key,
-			  attr->val.str);
-		  break;
-		case IVL_ATT_NUM:
-		  fprintf(out, "    (* %s = %ld *)\n", attr->key,
-			  attr->val.num);
-		  break;
-	    }
+	    show_attribute(attr,4);
       }
 
       show_statement(ivl_process_stmt(net), 4);
@@ -801,12 +968,22 @@ static void show_logic(ivl_net_logic_t net)
 		    name,
 		    ivl_nexus_name(ivl_logic_pin(net, 0)));
 	    break;
+
+	  case IVL_LO_EEQ:
+	    fprintf(out, "  EEQ %s (%s", name,
+		    ivl_nexus_name(ivl_logic_pin(net,0)));
+	    break;
+
 	  case IVL_LO_NOT:
 	    fprintf(out, "  not #(%u) %s (%s",
 		    ivl_logic_delay(net, 0),
 		    name,
 		    ivl_nexus_name(ivl_logic_pin(net, 0)));
 	    break;
+	  case IVL_LO_NOR:
+	    fprintf(out, "  nor %s (%s", name,
+		    ivl_nexus_name(ivl_logic_pin(net, 0)));
+	    break;
 	  case IVL_LO_OR:
 	    fprintf(out, "  or %s (%s", name,
 		    ivl_nexus_name(ivl_logic_pin(net, 0)));
@@ -822,7 +999,8 @@ static void show_logic(ivl_net_logic_t net)
 	    break;
 
 	  default:
-	    fprintf(out, "  unsupported gate %s (%s", name,
+	    fprintf(out, "  unsupported gate(%u) %s (%s",
+		    ivl_logic_type(net), name,
 		    ivl_nexus_name(ivl_logic_pin(net, 0)));
 	    break;
       }
@@ -953,6 +1131,39 @@ int target_design(ivl_design_t des)
 
 /*
  * $Log: stub.c,v $
+ * Revision 1.90.2.11  2006/09/15 23:56:05  steve
+ *  Special handling of exploded memory arguments.
+ *
+ * Revision 1.90.2.10  2006/07/23 19:42:35  steve
+ *  Handle statement output override better in blocks.
+ *
+ * Revision 1.90.2.9  2006/04/16 19:26:41  steve
+ *  Fix handling of exploded memories with partial or missing resets.
+ *
+ * Revision 1.90.2.8  2006/03/26 23:09:25  steve
+ *  Handle asynchronous demux/bit replacements.
+ *
+ * Revision 1.90.2.7  2006/03/12 07:34:20  steve
+ *  Fix the memsynth1 case.
+ *
+ * Revision 1.90.2.6  2006/02/25 05:03:30  steve
+ *  Add support for negedge FFs by using attributes.
+ *
+ * Revision 1.90.2.5  2006/02/19 00:11:34  steve
+ *  Handle synthesis of FF vectors with l-value decoder.
+ *
+ * Revision 1.90.2.4  2006/01/21 21:42:33  steve
+ *  When mux has wide select but sparse choices, use 1hot translation.
+ *
+ * Revision 1.90.2.3  2006/01/14 20:13:22  steve
+ *  Show synchronous set/clr of FF.
+ *
+ * Revision 1.90.2.2  2005/08/28 17:37:28  steve
+ *  Dump CMP_GE devices.
+ *
+ * Revision 1.90.2.1  2005/08/21 22:27:57  steve
+ *  Display NOR gates.
+ *
  * Revision 1.90  2004/10/04 01:10:57  steve
  *  Clean up spurious trailing white space.
  *
