@@ -17,7 +17,7 @@
  *    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
  */
 #ifdef HAVE_CVS_IDENT
-#ident "$Id: eval_expr.c,v 1.110 2004/10/04 01:10:57 steve Exp $"
+#ident "$Id: eval_expr.c,v 1.110.2.2 2007/02/26 19:51:40 steve Exp $"
 #endif
 
 # include  "vvp_priv.h"
@@ -101,7 +101,7 @@ unsigned long get_number_immediate(ivl_expr_t ex)
 
 /*
  * The STUFF_OK_XZ bit is true if the output is going to be further
- * processed so that x and z values are equivilent. This may allow for
+ * processed so that x and z values are equivalent. This may allow for
  * new optimizations.
  */
 static struct vector_info draw_eq_immediate(ivl_expr_t exp, unsigned ewid,
@@ -443,7 +443,7 @@ static struct vector_info draw_binary_expr_lor(ivl_expr_t exp, unsigned wid)
       lv = draw_eval_expr(le, STUFF_OK_XZ);
 
 	/* if the left operand has width, then evaluate the single-bit
-	   logical equivilent. */
+	   logical equivalent. */
       if ((lv.base >= 4) && (lv.wid > 1)) {
 	    struct vector_info tmp;
 	    clr_vector(lv);
@@ -457,7 +457,7 @@ static struct vector_info draw_binary_expr_lor(ivl_expr_t exp, unsigned wid)
       rv = draw_eval_expr(re, STUFF_OK_XZ);
 
 	/* if the right operand has width, then evaluate the single-bit
-	   logical equivilent. */
+	   logical equivalent. */
       if ((rv.base >= 4) && (rv.wid > 1)) {
 	    struct vector_info tmp;
 	    clr_vector(rv);
@@ -1002,7 +1002,7 @@ static struct vector_info draw_binary_expr_arith(ivl_expr_t exp, unsigned wid)
 
 	/* The arithmetic instructions replace the left operand with
 	   the result. If the left operand is a replicated constant,
-	   then I need to make a writeable copy so that the
+	   then I need to make a writable copy so that the
 	   instruction can operate. */
       if (lv.base < 4) {
 	    struct vector_info tmp;
@@ -1160,7 +1160,7 @@ static struct vector_info draw_bitsel_expr(ivl_expr_t exp, unsigned wid)
 
 /*
  * The concatenation operator is evaluated by evaluating each sub-
- * expression, then copying it into the continguous vector of the
+ * expression, then copying it into the contiguous vector of the
  * result. Do this until the result vector is filled.
  */
 static struct vector_info draw_concat_expr(ivl_expr_t exp, unsigned wid)
@@ -1621,7 +1621,7 @@ static struct vector_info draw_select_expr(ivl_expr_t exp, unsigned wid)
       clr_vector(shiv);
 
 	/* If the subv result is a magic constant, then make a copy in
-	   writeable vector space and work from there instead. */
+	   writable vector space and work from there instead. */
       if (subv.base < 4) {
 	    res.base = allocate_vector(subv.wid);
 	    res.wid = subv.wid;
@@ -1841,6 +1841,40 @@ static struct vector_info draw_ufunc_expr(ivl_expr_t exp, unsigned wid)
       return res;
 }
 
+static struct vector_info draw_ulong_expr(ivl_expr_t exp, unsigned wid)
+{
+      unsigned long idx;
+      struct vector_info res;
+      unsigned long uval = ivl_expr_uvalue(exp);
+
+      if (uval == 0) {
+	    res.wid = wid;
+	    res.base = 0;
+	    return res;
+      }
+
+      res.base = allocate_vector(wid);
+      res.wid = wid;
+
+      idx = 0;
+      while (idx < wid) {
+	    unsigned long cnt;
+
+	    int bit = 1 & (uval >> idx);
+	    for (cnt = 1 ;  idx+cnt < wid ;  cnt += 1) {
+		  int tmp = 1 & (uval >> (idx+cnt));
+		  if (tmp != bit)
+			break;
+	    }
+
+	    fprintf(vvp_out, "   %%mov %u, %d, %lu;\n",
+		    res.base+idx, bit, cnt);
+	    idx += cnt;
+      }
+
+      return res;
+}
+
 static struct vector_info draw_unary_expr(ivl_expr_t exp, unsigned wid)
 {
       struct vector_info res;
@@ -2120,6 +2154,10 @@ struct vector_info draw_eval_expr_wid(ivl_expr_t exp, unsigned wid,
 	    res = draw_ufunc_expr(exp, wid);
 	    break;
 
+	  case IVL_EX_ULONG:
+	    res = draw_ulong_expr(exp, wid);
+	    break;
+
 	  case IVL_EX_UNARY:
 	    res = draw_unary_expr(exp, wid);
 	    break;
@@ -2139,6 +2177,12 @@ struct vector_info draw_eval_expr(ivl_expr_t exp, int stuff_ok_flag)
 
 /*
  * $Log: eval_expr.c,v $
+ * Revision 1.110.2.2  2007/02/26 19:51:40  steve
+ *  Spelling fixes (larry doolittle)
+ *
+ * Revision 1.110.2.1  2006/03/12 07:34:20  steve
+ *  Fix the memsynth1 case.
+ *
  * Revision 1.110  2004/10/04 01:10:57  steve
  *  Clean up spurious trailing white space.
  *
