@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1999 Stephen Williams (steve@icarus.com)
+ * Copyright (c) 1999-2003 Stephen Williams (steve@icarus.com)
  *
  *    This source code is free software; you can redistribute it
  *    and/or modify it in source code form under the terms of the GNU
@@ -16,11 +16,11 @@
  *    along with this program; if not, write to the Free Software
  *    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
  */
-#if !defined(WINNT) && !defined(macintosh)
-#ident "$Id: sys_vcd.c,v 1.26 2001/10/26 02:29:10 steve Exp $"
+#ifdef HAVE_CVS_IDENT
+#ident "$Id: sys_vcd.c,v 1.56 2004/10/04 01:10:58 steve Exp $"
 #endif
 
-# include "config.h"
+# include "sys_priv.h"
 
 /*
  * This file contains the implementations of the VCD related
@@ -36,7 +36,9 @@
 #ifdef HAVE_MALLOC_H
 # include  <malloc.h>
 #endif
+# include  "vcd_priv.h"
 
+static char*dump_path = 0;
 static FILE*dump_file = 0;
 
 static const char*units_names[] = {
@@ -53,7 +55,9 @@ struct vcd_info {
       vpiHandle cb;
       struct t_vpi_time time;
       const char*ident;
-      struct vcd_info*next;
+      struct vcd_info* next;
+      struct vcd_info* dmp_next;
+      int scheduled;
 };
 
 
@@ -63,20 +67,21 @@ static void gen_new_vcd_id(void)
 {
       static unsigned value = 0;
       unsigned v = ++value;
-      int i;
+      unsigned int i;
 
       for (i=0; i < sizeof(vcdid)-1; i++) {
            vcdid[i] = (char)((v%94)+33); /* for range 33..126 */
            v /= 94;
            if(!v) {
-                 vcdid[i+1] = '\0'; 
+                 vcdid[i+1] = '\0';
                  break;
            }
       }
 }
 
-static struct vcd_info*vcd_list = 0;
-unsigned long vcd_cur_time = 0;
+static struct vcd_info *vcd_list = 0;
+static struct vcd_info *vcd_dmp_list = 0;
+PLI_UINT64 vcd_cur_time = 0;
 static int dump_is_off = 0;
 
 static char *truncate_bitvec(char *s)
@@ -88,13 +93,13 @@ static char *truncate_bitvec(char *s)
 	    return s;
       else
 	    s += 1;
-            
-      for(;;s++) { 
+
+      for(;;s++) {
 	    l=r; r=*s;
 	    if(!r) return (s-1);
-                 
+
 	    if(l!=r)
-		  return(((l=='0')&&(r='1'))?s:s-1);
+		  return(((l=='0')&&(r=='1'))?s:s-1);
 
       }
 }
@@ -103,7 +108,12 @@ static void show_this_item(struct vcd_info*info)
 {
       s_vpi_value value;
 
-      if (vpi_get(vpiSize, info->item) == 1) {
+      if (vpi_get(vpiType, info->item) == vpiRealVar) {
+	    value.format = vpiRealVal;
+	    vpi_get_value(info->item, &value);
+	    fprintf(dump_file, "r%.16g %s\n", value.value.real, info->ident);
+
+      } else if (vpi_get(vpiSize, info->item) == 1) {
 	    value.format = vpiBinStrVal;
 	    vpi_get_value(info->item, &value);
 	    fprintf(dump_file, "%s%s\n", value.value.str, info->ident);
@@ -119,7 +129,10 @@ static void show_this_item(struct vcd_info*info)
 
 static void show_this_item_x(struct vcd_info*info)
 {
-      if (vpi_get(vpiSize, info->item) == 1) {
+      if (vpi_get(vpiType, info->item) == vpiRealVar) {
+	      /* Some tools dump nothing here...? */
+	    fprintf(dump_file, "rNaN %s\n", info->ident);
+      } else if (vpi_get(vpiSize, info->item) == 1) {
 	    fprintf(dump_file, "x%s\n", info->ident);
       } else {
 	    fprintf(dump_file, "bx %s\n", info->ident);
@@ -131,77 +144,11 @@ static void show_this_item_x(struct vcd_info*info)
  * managed qsorted list of scope names for duplicates bsearching
  */
 
-struct vcd_names_s {
-      const char *name;
-      struct vcd_names_s *next;
-};
-
-static struct vcd_names_s *vcd_names_list;
-static const char **vcd_names_sorted;
-static int listed_names, sorted_names;
-
-inline static void vcd_names_add(const char *name)
-{
-      struct vcd_names_s *nl = (struct vcd_names_s *)
-	    malloc(sizeof(struct vcd_names_s));
-      assert(nl);
-      nl->name = name;
-      nl->next = vcd_names_list;
-      vcd_names_list = nl;
-      listed_names ++;
-}
-
-static int vcd_names_compare(const void *s1, const void *s2)
-{
-      const char *v1 = *(const char **) s1;
-      const char *v2 = *(const char **) s2;
-
-      return strcmp(v1, v2);
-}
-
-static const char *vcd_names_search(const char *key)
-{
-      const char **v = (const char **)
-	    bsearch(&key, 
-		    vcd_names_sorted, sorted_names,
-		    sizeof(const char *), vcd_names_compare );
-      
-      return(v ? *v : NULL);
-}
-
-void vcd_names_sort(void)
-{
-      if (listed_names) {
-	    struct vcd_names_s *r; 
-	    const char **l;
-	    
-	    sorted_names += listed_names;
-	    vcd_names_sorted = (const char **) 
-		  realloc(vcd_names_sorted, 
-			  sorted_names*(sizeof(const char *)));
-	    assert(vcd_names_sorted);
-	    
-	    l = vcd_names_sorted + sorted_names - listed_names;
-	    listed_names = 0;
-
-	    r = vcd_names_list;
-	    vcd_names_list = 0x0;
-
-	    while (r) {
-		  struct vcd_names_s *rr = r;
-		  r = rr->next;
-		  *(l++) = rr->name;
-		  free(rr);
-	    }
-	    
-	    qsort(vcd_names_sorted, sorted_names, 
-		  sizeof(const char **), vcd_names_compare);
-      }
-}
+struct vcd_names_list_s vcd_tab = { 0 };
 
 
 static int dumpvars_status = 0; /* 0:fresh 1:cb installed, 2:callback done */
-static unsigned long dumpvars_time;
+static PLI_UINT64 dumpvars_time;
 inline static int dump_header_pending(void)
 {
       return dumpvars_status != 2;
@@ -227,29 +174,45 @@ static void vcd_checkpoint_x()
 	    show_this_item_x(cur);
 }
 
-static int variable_cb(p_cb_data cause)
+static int variable_cb_2(p_cb_data cause)
 {
-      unsigned long now = cause->time->low;
-      struct t_cb_data cb;
-      struct vcd_info*info = (struct vcd_info*)cause->user_data;
-
-	/* Reschedule this event so that it happens for the next
-	   trigger on this variable. */
-      cb = *cause;
-      vpi_register_cb(&cb);
-
-      if (dump_is_off)
-	    return 0;
-
-      if (dump_header_pending())
-	    return 0;
+      struct vcd_info* info = vcd_dmp_list;
+      PLI_UINT64 now = timerec_to_time64(cause->time);
 
       if (now != vcd_cur_time) {
-	    fprintf(dump_file, "#%lu\n", now);
+	    fprintf(dump_file, "#%" PLI_UINT64_FMT "\n", now);
 	    vcd_cur_time = now;
       }
 
-      show_this_item(info);
+      do {
+           show_this_item(info);
+           info->scheduled = 0;
+      } while ((info = info->dmp_next) != 0);
+
+      vcd_dmp_list = 0;
+
+      return 0;
+}
+
+static int variable_cb_1(p_cb_data cause)
+{
+      struct t_cb_data cb;
+      struct vcd_info*info = (struct vcd_info*)cause->user_data;
+
+      if (dump_is_off) 		 return 0;
+      if (dump_header_pending()) return 0;
+      if (info->scheduled)       return 0;
+
+      if (!vcd_dmp_list) {
+          cb = *cause;
+          cb.reason = cbReadOnlySynch;
+          cb.cb_rtn = variable_cb_2;
+          vpi_register_cb(&cb);
+      }
+
+      info->scheduled = 1;
+      info->dmp_next  = vcd_dmp_list;
+      vcd_dmp_list    = info;
 
       return 0;
 }
@@ -261,13 +224,13 @@ static int dumpvars_cb(p_cb_data cause)
 
       dumpvars_status = 2;
 
-      dumpvars_time = cause->time->low;
+      dumpvars_time = timerec_to_time64(cause->time);
       vcd_cur_time = dumpvars_time;
 
       fprintf(dump_file, "$enddefinitions $end\n");
 
       if (!dump_is_off) {
-	    fprintf(dump_file, "#%lu\n", dumpvars_time);
+	    fprintf(dump_file, "#%" PLI_UINT64_FMT "\n", dumpvars_time);
 	    fprintf(dump_file, "$dumpvars\n");
 	    vcd_checkpoint();
 	    fprintf(dump_file, "$end\n");
@@ -285,9 +248,9 @@ inline static int install_dumpvars_callback(void)
 	    return 0;
 
       if (dumpvars_status == 2) {
-	    vpi_mcd_printf(6, "VCD Error:"
+	    vpi_mcd_printf(1, "VCD Error:"
 			   " $dumpvars ignored,"
-			   " previously called at simtime %lu\n",
+			   " previously called at simtime %" PLI_UINT64_FMT "\n",
 			   dumpvars_time);
 	    return 1;
       }
@@ -308,6 +271,7 @@ inline static int install_dumpvars_callback(void)
 static int sys_dumpoff_calltf(char*name)
 {
       s_vpi_time now;
+      PLI_UINT64 now64;
 
       if (dump_is_off)
 	    return 0;
@@ -320,10 +284,13 @@ static int sys_dumpoff_calltf(char*name)
       if (dump_header_pending())
 	    return 0;
 
+      now.type = vpiSimTime;
       vpi_get_time(0, &now);
-      if (now.low > vcd_cur_time)
-	    fprintf(dump_file, "#%u\n", now.low);
-      vcd_cur_time = now.low;
+      now64 = timerec_to_time64(&now);
+
+      if (now64 > vcd_cur_time)
+	    fprintf(dump_file, "#%" PLI_UINT64_FMT "\n", now64);
+      vcd_cur_time = now64;
 
       fprintf(dump_file, "$dumpoff\n");
       vcd_checkpoint_x();
@@ -335,6 +302,7 @@ static int sys_dumpoff_calltf(char*name)
 static int sys_dumpon_calltf(char*name)
 {
       s_vpi_time now;
+      PLI_UINT64 now64;
 
       if (!dump_is_off)
 	    return 0;
@@ -347,10 +315,13 @@ static int sys_dumpon_calltf(char*name)
       if (dump_header_pending())
 	    return 0;
 
+      now.type = vpiSimTime;
       vpi_get_time(0, &now);
-      if (now.low > vcd_cur_time)
-	    fprintf(dump_file, "#%u\n", now.low);
-      vcd_cur_time = now.low;
+      now64 = timerec_to_time64(&now);
+
+      if (now64 > vcd_cur_time)
+	    fprintf(dump_file, "#%" PLI_UINT64_FMT "\n", now64);
+      vcd_cur_time = now64;
 
       fprintf(dump_file, "$dumpon\n");
       vcd_checkpoint();
@@ -362,6 +333,7 @@ static int sys_dumpon_calltf(char*name)
 static int sys_dumpall_calltf(char*name)
 {
       s_vpi_time now;
+      PLI_UINT64 now64;
 
       if (dump_file == 0)
 	    return 0;
@@ -369,9 +341,12 @@ static int sys_dumpall_calltf(char*name)
       if (dump_header_pending())
 	    return 0;
 
+      now.type = vpiSimTime;
       vpi_get_time(0, &now);
-      if (now.low > vcd_cur_time)
-	    fprintf(dump_file, "#%u\n", now.low);
+      now64 = timerec_to_time64(&now);
+
+      if (now64 > vcd_cur_time)
+	    fprintf(dump_file, "#%" PLI_UINT64_FMT "\n", now64);
       vcd_cur_time = now.low;
 
       fprintf(dump_file, "$dumpall\n");
@@ -381,14 +356,18 @@ static int sys_dumpall_calltf(char*name)
       return 0;
 }
 
-static void open_dumpfile(const char*path)
+static void open_dumpfile(void)
 {
-      dump_file = fopen(path, "w");
+      if (dump_path == 0) {
+	    dump_path = strdup("dump.vcd");
+      }
+
+      dump_file = fopen(dump_path, "w");
 
       if (dump_file == 0) {
-	    vpi_mcd_printf(6, 
-			   "VCD Error: Unable to open %s for output.\n", 
-			   path);
+	    vpi_mcd_printf(1,
+			   "VCD Error: Unable to open %s for output.\n",
+			   dump_path);
 	    return;
       } else {
 	    int prec = vpi_get(vpiTimePrecision, 0);
@@ -396,10 +375,10 @@ static void open_dumpfile(const char*path)
 	    unsigned udx = 0;
 	    time_t walltime;
 
-	    vpi_mcd_printf(4, 
-			   "VCD info: dumpfile %s opened for output.\n", 
-			   path);
-	    
+	    vpi_mcd_printf(1,
+			   "VCD info: dumpfile %s opened for output.\n",
+			   dump_path);
+
 	    time(&walltime);
 
 	    assert(prec >= -15);
@@ -424,22 +403,22 @@ static void open_dumpfile(const char*path)
       }
 }
 
-static int sys_dumpfile_calltf(char*name)
+static int sys_dumpfile_compiletf(char*name)
 {
-      char*path;
-
       vpiHandle sys = vpi_handle(vpiSysTfCall, 0);
       vpiHandle argv = vpi_iterate(vpiArgument, sys);
       vpiHandle item;
 
+      char*path;
+
       if (argv && (item = vpi_scan(argv))) {
 	    s_vpi_value value;
 
 	    if (vpi_get(vpiType, item) != vpiConstant
 		|| vpi_get(vpiConstType, item) != vpiStringConst) {
-		  vpi_mcd_printf(6, 
+		  vpi_mcd_printf(1,
 				 "VCD Error:"
-				 " %s parameter must be a string constant\n", 
+				 " %s parameter must be a string constant\n",
 				 name);
 		  return 0;
 	    }
@@ -451,80 +430,32 @@ static int sys_dumpfile_calltf(char*name)
 	    vpi_free_object(argv);
 
       } else {
-	    path = strdup("dumpfile.vcd");
+	    path = strdup("dump.vcd");
       }
 
-      assert(dump_file == 0);
-      open_dumpfile(path);
+      if (dump_path) {
+	    vpi_mcd_printf(1, "VCD Warning:"
+			   " Overriding dumpfile path %s with %s\n",
+			   dump_path, path);
+	    free(dump_path);
+      }
 
-      free(path);
+      dump_path = path;
 
       return 0;
 }
 
-/* 
-   Nexus Id cache
-
-   In structural models, many signals refer to the same nexus.
-   Some structural models also have very many signals.  This cache
-   saves nexus_id - vcd_id pairs, and reuses the vcd_id when a signal 
-   refers to a nexus that is already dumped.
-
-   The new signal will be listed as a $var, but no callback 
-   will be installed.  This saves considerable CPU time and leads 
-   to smalle VCD files.
-
-   The _vpiNexusId is a private (int) property of IVL simulators.
-*/
-
-struct vcd_id_s 
+static int sys_dumpfile_calltf(char*name)
 {
-  const char *id;
-  struct vcd_id_s *next;
-  int nex;
-};
-
-static inline unsigned ihash(int nex)
-{
-  unsigned a = nex;
-  a ^= a>>16;
-  a ^= a>>8;
-  return a & 0xff;
-}
-
-static struct vcd_id_s **vcd_ids;
-
-inline static const char *find_nexus_ident(int nex)
-{
-      struct vcd_id_s *bucket;
-      
-      if (!vcd_ids) {
-	    vcd_ids = (struct vcd_id_s **)
-		  calloc(256, sizeof(struct vcd_id_s*));
-	    assert(vcd_ids);
-      }
-
-      bucket = vcd_ids[ihash(nex)];
-      while (bucket) {
-	    if (nex == bucket->nex)
-		  return bucket->id;
-	    bucket = bucket->next;
-      }
-
       return 0;
 }
 
-inline static void set_nexus_ident(int nex, const char *id)
+static int sys_dumpflush_calltf(char*name)
 {
-      struct vcd_id_s *bucket;
+      if (dump_file)
+	    fflush(dump_file);
 
-      assert(vcd_ids);
-
-      bucket = (struct vcd_id_s *) malloc(sizeof(struct vcd_id_s));
-      bucket->next = vcd_ids[ihash(nex)];
-      bucket->id = id;
-      bucket->nex = nex;
-      vcd_ids[ihash(nex)] = bucket;
+      return 0;
 }
 
 static void scan_item(unsigned depth, vpiHandle item, int skip)
@@ -537,16 +468,42 @@ static void scan_item(unsigned depth, vpiHandle item, int skip)
       const char* ident;
       int nexus_id;
 
+      /* list of types to iterate upon */
+      int i;
+      static int types[] = {
+	    /* Value */
+	    vpiNet,
+	    vpiReg,
+	    vpiVariables,
+	    /* Scope */
+	    vpiFunction,
+	    vpiModule,
+	    vpiNamedBegin,
+	    vpiNamedFork,
+	    vpiTask,
+	    -1
+      };
+
       switch (vpi_get(vpiType, item)) {
 
+	  case vpiMemory:
+	      /* don't know how to watch memories. */
+	    break;
+
+	  case vpiNamedEvent:
+	      /* There is nothing in named events to dump. */
+	    break;
+
 	  case vpiNet:  type = "wire";    if(0){
+	  case vpiIntegerVar:
+	  case vpiTimeVar:
 	  case vpiReg:  type = "reg";    }
 
 	    if (skip)
 		  break;
-	    
+
 	    name = vpi_get_str(vpiName, item);
-	    
+
 	    nexus_id = vpi_get(_vpiNexusId, item);
 
 	    if (nexus_id) {
@@ -554,38 +511,85 @@ static void scan_item(unsigned depth, vpiHandle item, int skip)
 	    } else {
 		  ident = 0;
 	    }
-	    
+
 	    if (!ident) {
 		  ident = strdup(vcdid);
 		  gen_new_vcd_id();
-		  
+
 		  if (nexus_id)
 			set_nexus_ident(nexus_id, ident);
-		  
+
 		  info = malloc(sizeof(*info));
 
 		  info->time.type = vpiSimTime;
 		  info->item  = item;
 		  info->ident = ident;
+		  info->scheduled = 0;
 
 		  cb.time      = &info->time;
 		  cb.user_data = (char*)info;
 		  cb.value     = NULL;
 		  cb.obj       = item;
 		  cb.reason    = cbValueChange;
-		  cb.cb_rtn    = variable_cb;
+		  cb.cb_rtn    = variable_cb_1;
 
-		  info->next  = vcd_list;
+
+		  info->next      = vcd_list;
+		  info->dmp_next  = 0;
 		  vcd_list    = info;
 
 		  info->cb    = vpi_register_cb(&cb);
 	    }
-	    
-	    fprintf(dump_file, "$var %s %u %s %s $end\n",
+
+	    fprintf(dump_file, "$var %s %u %s %s",
 		    type, vpi_get(vpiSize, item), ident,
 		    name);
+      /* FIXME
+	    if (vpi_get(vpiVector, item)
+      */
+	    if (vpi_get(vpiSize, item) > 1
+		|| vpi_get(vpiLeftRange, item) != 0) {
+		  fprintf(dump_file, "[%i:%i]", vpi_get(vpiLeftRange, item),
+			  vpi_get(vpiRightRange, item));
+	    }
+	    fprintf(dump_file, " $end\n");
 	    break;
-	    
+
+	  case vpiRealVar:
+
+	    if (skip)
+		  break;
+
+	      /* Declare the variable in the VCD file. */
+	    name = vpi_get_str(vpiName, item);
+	    ident = strdup(vcdid);
+	    gen_new_vcd_id();
+	    fprintf(dump_file, "$var real 1 %s %s $end\n",
+		    ident, name);
+
+	      /* Add a callback for the variable. */
+	    info = malloc(sizeof(*info));
+
+	    info->time.type = vpiSimTime;
+	    info->item  = item;
+	    info->ident = ident;
+	    info->scheduled = 0;
+
+	    cb.time      = &info->time;
+	    cb.user_data = (char*)info;
+	    cb.value     = NULL;
+	    cb.obj       = item;
+	    cb.reason    = cbValueChange;
+	    cb.cb_rtn    = variable_cb_1;
+
+	    info->next  = vcd_list;
+	    info->dmp_next  = 0;
+	    vcd_list    = info;
+
+	    info->cb    = vpi_register_cb(&cb);
+
+	    break;
+
 	  case vpiModule:      type = "module";      if(0){
 	  case vpiNamedBegin:  type = "begin";      }if(0){
 	  case vpiTask:        type = "task";       }if(0){
@@ -599,18 +603,18 @@ static void scan_item(unsigned depth, vpiHandle item, int skip)
 		  const char* fullname =
 			vpi_get_str(vpiFullName, item);
 
-		  vpi_mcd_printf(4, 
+#if 0
+		  vpi_mcd_printf(1,
 				 "VCD info:"
 				 " scanning scope %s, %u levels\n",
 				 fullname, depth);
+#endif
+		  nskip = 0 != vcd_names_search(&vcd_tab, fullname);
 
-		  nskip = sorted_names && fullname
-			&& vcd_names_search(fullname);
-		  
-		  if (!nskip) 
-			vcd_names_add(fullname);
-		  else 
-		    vpi_mcd_printf(6,
+		  if (!nskip)
+			vcd_names_add(&vcd_tab, fullname);
+		  else
+		    vpi_mcd_printf(1,
 				   "VCD warning:"
 				   " ignoring signals"
 				   " in previously scanned scope %s\n",
@@ -619,23 +623,21 @@ static void scan_item(unsigned depth, vpiHandle item, int skip)
 		  name = vpi_get_str(vpiName, item);
 
 		  fprintf(dump_file, "$scope %s %s $end\n", type, name);
-		  
-		  argv = vpi_iterate(vpiInternalScope, item);
-		  if (argv) {
-			for (item = vpi_scan(argv) ;  
-			     item ;  
-			     item = vpi_scan(argv)) {
-			      
-			      scan_item(depth-1, item, nskip);
+
+		  for (i=0; types[i]>0; i++) {
+			vpiHandle hand;
+			argv = vpi_iterate(types[i], item);
+			while (argv && (hand = vpi_scan(argv))) {
+			      scan_item(depth-1, hand, nskip);
 			}
 		  }
-		  
+
 		  fprintf(dump_file, "$upscope $end\n");
 	    }
 	    break;
-	    
+
 	  default:
-	    vpi_mcd_printf(6,
+	    vpi_mcd_printf(1,
 			   "VCD Error: $dumpvars: Unsupported parameter "
 			   "type (%d)\n", vpi_get(vpiType, item));
       }
@@ -650,7 +652,7 @@ static int draw_scope(vpiHandle item)
       vpiHandle scope = vpi_handle(vpiScope, item);
       if (!scope)
 	    return 0;
-      
+
       depth = 1 + draw_scope(scope);
       name = vpi_get_str(vpiName, scope);
 
@@ -661,12 +663,54 @@ static int draw_scope(vpiHandle item)
 	  case vpiNamedFork:   type = "fork";       break;
       	  default:             type = "module";     break;
       }
-      
+
       fprintf(dump_file, "$scope %s %s $end\n", type, name);
 
       return depth;
 }
 
+/*
+ * This function is also used in sys_lxt to check the arguments of the
+ * lxt variant of $dumpvars.
+ */
+int sys_vcd_dumpvars_compiletf(char*name)
+{
+      vpiHandle sys   = vpi_handle(vpiSysTfCall, 0);
+      vpiHandle argv  = vpi_iterate(vpiArgument, sys);
+      vpiHandle tmp;
+
+      if (argv == 0)
+	    return 0;
+
+      tmp = vpi_scan(argv);
+      assert(tmp);
+
+      switch (vpi_get(vpiType, tmp)) {
+	  case vpiConstant:
+	    if (vpi_get(vpiConstType, tmp) == vpiStringConst) {
+		  vpi_printf("ERROR: %s argument must be "
+			     "a number constant.\n", name);
+		  vpi_control(vpiFinish, 1);
+	    }
+	    break;
+
+	  case vpiNet:
+	  case vpiReg:
+	  case vpiIntegerVar:
+	  case vpiMemoryWord:
+	    break;
+
+	  default:
+	    vpi_printf("ERROR: %s argument must be "
+		       "a number constant.\n", name);
+	    vpi_control(vpiFinish, 1);
+	    break;
+      }
+
+      vpi_free_object(argv);
+      return 0;
+}
+
 static int sys_dumpvars_calltf(char*name)
 {
       unsigned depth;
@@ -676,7 +720,7 @@ static int sys_dumpvars_calltf(char*name)
       vpiHandle argv;
 
       if (dump_file == 0) {
-	    open_dumpfile("dumpfile.vcd");
+	    open_dumpfile();
 	    if (dump_file == 0)
 		  return 0;
       }
@@ -693,6 +737,7 @@ static int sys_dumpvars_calltf(char*name)
 		case vpiConstant:
 		case vpiNet:
 		case vpiReg:
+		case vpiIntegerVar:
 		case vpiMemoryWord:
 		  value.format = vpiIntVal;
 		  vpi_get_value(item, &value);
@@ -724,9 +769,9 @@ static int sys_dumpvars_calltf(char*name)
 
 	    int dep = draw_scope(item);
 
-	    vcd_names_sort();
+	    vcd_names_sort(&vcd_tab);
 	    scan_item(depth, item, 0);
-	    
+
 	    while (dep--) {
 		  fprintf(dump_file, "$upscope $end\n");
 	    }
@@ -766,15 +811,23 @@ void sys_vcd_register()
       tf_data.type      = vpiSysTask;
       tf_data.tfname    = "$dumpfile";
       tf_data.calltf    = sys_dumpfile_calltf;
-      tf_data.compiletf = 0;
+      tf_data.compiletf = sys_dumpfile_compiletf;
       tf_data.sizetf    = 0;
       tf_data.user_data = "$dumpfile";
       vpi_register_systf(&tf_data);
 
+      tf_data.type      = vpiSysTask;
+      tf_data.tfname    = "$dumpflush";
+      tf_data.calltf    = sys_dumpflush_calltf;
+      tf_data.compiletf = 0;
+      tf_data.sizetf    = 0;
+      tf_data.user_data = "$dumpflush";
+      vpi_register_systf(&tf_data);
+
       tf_data.type      = vpiSysTask;
       tf_data.tfname    = "$dumpvars";
       tf_data.calltf    = sys_dumpvars_calltf;
-      tf_data.compiletf = 0;
+      tf_data.compiletf = sys_vcd_dumpvars_compiletf;
       tf_data.sizetf    = 0;
       tf_data.user_data = "$dumpvars";
       vpi_register_systf(&tf_data);
@@ -782,88 +835,40 @@ void sys_vcd_register()
 
 /*
  * $Log: sys_vcd.c,v $
- * Revision 1.26  2001/10/26 02:29:10  steve
- *  const/non-const warnings. (Stephan Boettcher)
+ * Revision 1.56  2004/10/04 01:10:58  steve
+ *  Clean up spurious trailing white space.
  *
- * Revision 1.25  2001/10/25 04:19:53  steve
- *  VPI support for callback to return values.
+ * Revision 1.55  2004/02/15 20:46:01  steve
+ *  Add the $dumpflush function
  *
- * Revision 1.24  2001/10/15 01:50:23  steve
- *  Include scope information in VCD output.
+ * Revision 1.54  2004/02/15 03:17:15  steve
+ *  dumpfile selects file at compiletf time.
  *
- * Revision 1.23  2001/10/14 18:32:06  steve
- *  More coverage of $dump related commands.
+ * Revision 1.53  2004/01/21 01:22:53  steve
+ *  Give the vip directory its own configure and vpi_config.h
  *
- * Revision 1.22  2001/10/08 23:33:00  steve
- *  Fix pr283: signal values before enddefinitions in vcd. (Stephan Boettcher)
+ * Revision 1.52  2003/12/19 01:27:10  steve
+ *  Fix various unsigned compare warnings.
  *
- * Revision 1.21  2001/09/30 05:18:46  steve
- *  Reduce VCD output by removing duplicates. (Stephan Boettcher)
+ * Revision 1.51  2003/10/29 03:28:27  steve
+ *  Add the PLU_UINT64_FMT string for formatting output.
  *
- * Revision 1.20  2001/07/25 03:10:50  steve
- *  Create a config.h.in file to hold all the config
- *  junk, and support gcc 3.0. (Stephan Boettcher)
+ * Revision 1.50  2003/10/29 03:23:12  steve
+ *  Portably handle time format of VCD prints.
  *
- * Revision 1.19  2001/07/16 18:53:16  steve
- *  Cut off scope iteration when depth runs out.
+ * Revision 1.49  2003/10/08 23:14:29  steve
+ *  TIME_FMT does not include the % character.
  *
- * Revision 1.18  2001/06/29 00:42:39  steve
- *  Get a private copy of the object name.
+ * Revision 1.48  2003/10/02 21:30:06  steve
+ *  Use configured TIME_FMT in vcd dump printf.
  *
- * Revision 1.17  2001/06/21 04:15:22  steve
- *  Add dumpon and dumpoff (Stephan Boettcher)
+ * Revision 1.47  2003/09/30 01:33:39  steve
+ *  dumpers must be aware of 64bit time.
  *
- * Revision 1.16  2001/01/23 18:50:26  steve
- *  Forgot to actually *open* the VCD output.
- *
- * Revision 1.15  2001/01/22 20:58:31  steve
- *  Support default dumpfiles.
- *
- * Revision 1.14  2001/01/01 08:10:35  steve
- *  Handle function scopes in dumpvars scn (PR#95)
- *
- * Revision 1.13  2000/11/01 06:05:44  steve
- *  VCD scans tasks (PR#35)
- *
- * Revision 1.12  2000/07/31 03:34:31  steve
- *  Report error when dumpfile is missing.
- *
- * Revision 1.11  2000/07/26 04:07:59  steve
- *  Get VCD timescale from design precision.
- *
- * Revision 1.10  2000/06/03 02:22:15  steve
- *  Interpret the depth paramter of dumpvars.
- *
- * Revision 1.9  2000/04/09 04:18:16  steve
- *  Catch duplicate $dumpvars of symbols (ajb)
- *
- * Revision 1.8  2000/04/08 05:28:39  steve
- *  Revamped VCD id generation and duplicates removal. (ajb)
- *
- * Revision 1.8  2000/04/06 21:00:00  ajb
- *  Revamped VCD id generation and duplicates removal.
- *
- * Revision 1.7  2000/02/23 02:56:56  steve
- *  Macintosh compilers do not support ident.
- *
- * Revision 1.6  2000/02/17 06:04:30  steve
- *  Fix overlap of identifiers when multiple modules used.
- *
- * Revision 1.5  2000/01/23 23:54:36  steve
- *  Compile time problems with vpi_user.h
- *
- * Revision 1.4  2000/01/20 06:04:55  steve
- *  $dumpall checkpointing in VCD dump.
- *
- * Revision 1.3  2000/01/13 04:48:50  steve
- *  Catch some parameter problems.
- *
- * Revision 1.2  1999/11/28 00:56:08  steve
- *  Build up the lists in the scope of a module,
- *  and get $dumpvars to scan the scope for items.
- *
- * Revision 1.1  1999/11/07 20:33:30  steve
- *  Add VCD output and related system tasks.
+ * Revision 1.46  2003/08/22 23:14:27  steve
+ *  Preserve variable ranges all the way to the vpi.
  *
+ * Revision 1.45  2003/08/06 18:24:55  steve
+ *  Fix error truncating bitvec in output.
  */
 
